# GemRB - Infinity Engine Emulator
# Copyright (C) 2003-2004 The GemRB Project
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
#


# GUIMG.py - scripts to control mage spells windows from the GUIMG winpack

###################################################

import GemRB
import GameCheck
import GUICommon
import CommonTables
import GUICommonWindows
import Spellbook
from GUICommon import BindControlCallbackParams
from GUIDefines import *
from ie_stats import *
from ie_spells import LS_MEMO

MageWindow = None
MageSpellLevel = 0

# bg2 stuff for handling triggers and contingencies
Sorcerer = None
OtherWindow = None
Exclusions = None
ContCond = None
ContTarg = None
SpellType = None
Level = 1

FlashResRef = "FLASHBR" if GameCheck.IsBG2OrEE () else "FLASH"

def ToggleSpellWindow (btn):
	global Sorcerer
	# added game check, since although sorcerers have almost no use for their spellbook, there's no other way to quickly check spell descriptions
	pc = GemRB.GameGetSelectedPCSingle ()
	Sorcerer = GameCheck.IsBG2OrEE () and Spellbook.HasSorcererBook (pc) > 0
	
	ToggleSpellWindow.Args = btn
	
	if Sorcerer:
		ToggleSorcererWindow (btn)
	else:
		ToggleMageWindow (btn)

def InitMageWindow (window):
	global MageWindow
	MageWindow = window

	Button = MageWindow.GetControl (1)
	Button.OnPress (MagePrevLevelPress)

	Button = MageWindow.GetControl (2)
	Button.OnPress (MageNextLevelPress)

	# contingencies
	Button = MageWindow.GetControl (55)
	if GameCheck.IsBG2EE ():
		Button = MageWindow.GetControl (59)
	if Button:
		Button.OnPress (OpenContingenciesWindow)
		pc = GemRB.GameGetSelectedPCSingle ()
		if GemRB.CountEffects (pc, "CastSpellOnCondition", -1, -1) + GemRB.CountEffects (pc, "Sequencer:Store", -1, -1) == 0:
			Button.SetState (IE_GUI_BUTTON_DISABLED)
			Button.SetText (None)
		else:
			Button.SetState (IE_GUI_BUTTON_ENABLED)
			Button.SetText (34586)

	#setup level buttons
	spellLevelOffset = None
	if GameCheck.IsBG2 ():
		spellLevelOffset = 56
	elif GameCheck.IsBG2EE ():
		spellLevelOffset = 61
	if spellLevelOffset:
		for i in range (9):
			Button = MageWindow.GetControl (spellLevelOffset + i)
			Button.OnPress (RefreshMageLevel)
			Button.SetFlags (IE_GUI_BUTTON_RADIOBUTTON, OP_OR)
			Button.SetVarAssoc ("MageSpellLevel", i)

	# Setup memorized spells buttons
	if not Sorcerer:
		for i in range (12):
			Button = MageWindow.GetControl (3 + i)
			color = {'r' : 0, 'g' : 0, 'b' :0, 'a' : 64}
			Button.SetBorder (0,color,0,1)
			Button.SetSprites ("SPELFRAM",0,0,0,0,0)
			Button.SetState (IE_GUI_BUTTON_LOCKED)
			Button.SetAnimation (None)
			Button.SetVarAssoc ("Memorized", i)

	# reposition from a 4x3 grid to the bottom or top
	# for mages that's the memoed spells, for sorcerers the known
	if GameCheck.IsBG2EE ():
		for i in range (12):
			Button = MageWindow.GetControl (3 + i + 24 * int(Sorcerer))
			row = i // 4
			col = i % 4
			width = 54 + 8 * (1 + int(Sorcerer)) # slot + padding
			# don't bother transposing 4x3 grid to 3x4 grid
			if Sorcerer:
				frame = Button.GetFrame ()
				x = frame["x"] - (314 - 166)
				y = frame["y"] - (536 - 234)
			else:
				x = 100 + row * width * 4 + col * width + 66 * int(Sorcerer)
				y = 644
			Button.SetPos (x, y)
			# the bam isn't centered, so let's cheat
			Button.SetFlags (IE_GUI_BUTTON_NO_IMAGE, OP_OR)

		# some extra slot button, sorcerer button and a label on a button
		MageWindow.DeleteControl (51)
		if Sorcerer:
			MageWindow.DeleteControl (59)
		else:
			MageWindow.DeleteControl (0x10000000 + 51)

	# Setup book spells buttons
	for i in range (GUICommon.GetGUISpellButtonCount()):
		Button = MageWindow.GetControl (27 + i)
		Button.SetFlags (IE_GUI_BUTTON_NO_IMAGE, OP_SET)
		Button.SetState (IE_GUI_BUTTON_LOCKED)
		Button.SetValue (i)

	UpdateMageWindow (MageWindow)
	return

def UpdateMageWindow (MageWindow):
	pc = GemRB.GameGetSelectedPCSingle ()
	spelltype = IE_SPELL_TYPE_WIZARD
	level = MageSpellLevel
	max_mem_cnt = GemRB.GetMemorizableSpellsCount (pc, spelltype, level, 1)
	
	CantCast = CommonTables.ClassSkills.GetValue (GUICommon.GetClassRowName (pc), "MAGESPELL") == "*"
	GUICommon.AdjustWindowVisibility (MageWindow, pc, CantCast)

	Label = MageWindow.GetControl (0x10000032)
	if GameCheck.IsBG2OrEE ():
		GemRB.SetToken ("SPELLLEVEL", str(level + 1))
		Label.SetText (10345)
	else:
		GemRB.SetToken ('LEVEL', str(level + 1))
		Label.SetText (12137)

	Name = GemRB.GetPlayerName (pc, 0)
	Label = MageWindow.GetControl (0x10000035)
	Label.SetText (Name)

	known_cnt = GemRB.GetKnownSpellsCount (pc, spelltype, level)
	mem_cnt = GemRB.GetMemorizedSpellsCount (pc, spelltype, level, False)
	true_mem_cnt = GemRB.GetMemorizedSpellsCount (pc, spelltype, level, True)
	if not Sorcerer:
		for i in range (12):
			Button = MageWindow.GetControl (3 + i)

			if i < mem_cnt:
				ms = GemRB.GetMemorizedSpell (pc, spelltype, level, i)
				Button.SetSpellIcon (ms['SpellResRef'], 0)
				if not GameCheck.IsBG2EE ():
					Button.SetFlags (IE_GUI_BUTTON_PICTURE, OP_SET)
				if ms['Flags']:
					Button.OnPress (OpenMageSpellUnmemorizeWindow)
				else:
					Button.OnPress (OnMageUnmemorizeSpell)
				spell = GemRB.GetSpell (ms['SpellResRef'])
				Button.OnRightPress(BindControlCallbackParams(OpenMageSpellInfoWindow, spell, Button.VarName))
				Button.EnableBorder (0, ms['Flags'] == 0)
				Button.SetTooltip (spell['SpellName'])
			else:
				if i < max_mem_cnt:
					Button.SetFlags (IE_GUI_BUTTON_NORMAL, OP_SET)
				else:
					Button.SetFlags (IE_GUI_BUTTON_NO_IMAGE, OP_SET)
				Button.OnPress (None)
				Button.OnRightPress (None)
				Button.SetTooltip ('')
				Button.EnableBorder (0, 0)
	else:
		label = MageWindow.GetControl (0x10000040)
		if known_cnt:
			# we give sorcerers all charges for all the spells, so some extra math is needed
			label.SetText (GemRB.GetString(61256) + " " + str(true_mem_cnt // known_cnt) + "/" + str(max_mem_cnt))
		else:
			label.SetText ("")
	
	btncount = GUICommon.GetGUISpellButtonCount()
	i = 0
	for i in range (known_cnt):
		Button = MageWindow.GetControl (27 + i)
		
		ks = GemRB.GetKnownSpell (pc, spelltype, level, i)
		Button.SetSpellIcon (ks['SpellResRef'], 0)
		Button.OnPress (OnMageMemorizeSpell)
		spell = GemRB.GetSpell (ks['SpellResRef'])
		spell['CurrentIndex'] = i # save it for later
		Button.OnRightPress (BindControlCallbackParams(OpenMageSpellInfoWindow, spell, Button.VarName))
		Button.SetTooltip (spell['SpellName'])

	if known_cnt == 0: i = -1
	for i in range (i + 1, btncount):
		Button = MageWindow.GetControl (27 + i)
		
		Button.SetFlags (IE_GUI_BUTTON_PICTURE, OP_NAND)
		Button.OnPress (None)
		Button.OnRightPress (None)
		Button.SetTooltip ('')
		Button.EnableBorder (0, 0)
	
	return

def MageSelectionChanged (oldwin):
	global Sorcerer
	# added game check, since although sorcerers have almost no use for their spellbook, there's no other way to quickly check spell descriptions
	
	UpdateMageWindow(oldwin)
	
	pc = GemRB.GameGetSelectedPCSingle ()
	Sorcerer = GameCheck.IsBG2OrEE () and Spellbook.HasSorcererBook (pc)

	if Sorcerer:
		OpenSorcererWindow (ToggleSpellWindow.Args)
	else:
		OpenMageWindow (ToggleSpellWindow.Args)

ToggleMageWindow = GUICommonWindows.CreateTopWinLoader(2, "GUIMG", GUICommonWindows.ToggleWindow, InitMageWindow, MageSelectionChanged, True)
OpenMageWindow = GUICommonWindows.CreateTopWinLoader(2, "GUIMG", GUICommonWindows.OpenWindowOnce, InitMageWindow, MageSelectionChanged, True)

ToggleSorcererWindow = GUICommonWindows.CreateTopWinLoader(8, "GUIMG", GUICommonWindows.ToggleWindow, InitMageWindow, MageSelectionChanged, True)
OpenSorcererWindow = GUICommonWindows.CreateTopWinLoader(8, "GUIMG", GUICommonWindows.OpenWindowOnce, InitMageWindow, MageSelectionChanged, True)

def MagePrevLevelPress ():
	global MageSpellLevel

	if MageSpellLevel > 0:
		MageSpellLevel = MageSpellLevel - 1
		UpdateMageWindow (MageWindow)
	return

def MageNextLevelPress ():
	global MageSpellLevel

	if MageSpellLevel < 8:
		MageSpellLevel = MageSpellLevel + 1
		UpdateMageWindow (MageWindow)
	return

def RefreshMageLevel ():
	global MageSpellLevel

	MageSpellLevel = GemRB.GetVar ("MageSpellLevel")
	UpdateMageWindow (MageWindow)
	return

def OpenMageSpellInfoWindow (spell, kind):
	Window = GemRB.LoadWindow (3, "GUIMG")

	#back
	Button = Window.GetControl (5)
	Button.SetText (15416)
	Button.OnPress (Window.Close)

	#erase
	Button = Window.GetControl (6)
	if Button:
		if kind == "Memorized" or Sorcerer:
			Button.OnPress (None)
			Button.SetFlags (IE_GUI_BUTTON_NO_IMAGE, OP_SET)
		else:
			Button.OnPress (lambda: OpenMageSpellRemoveWindow(Window, spell['CurrentIndex']))
			Button.SetText (63668)

	Label = Window.GetControl (0x0fffffff)
	Label.SetText (spell['SpellName'])

	Button = Window.GetControl (2)
	Button.SetSpellIcon (spell['SpellResRef'], 1)

	Text = Window.GetControl (3)
	Text.SetText (spell['SpellDesc'])

	Window.ShowModal (MODAL_SHADOW_GRAY)
	return

def OnMageMemorizeSpell (btn):
	pc = GemRB.GameGetSelectedPCSingle ()
	level = MageSpellLevel
	spelltype = IE_SPELL_TYPE_WIZARD
	Window = btn.Window

	def Complete():
		mem_cnt = GemRB.GetMemorizedSpellsCount (pc, spelltype, level, False)
		AnimBtn = Window.GetControl(mem_cnt + 2)
		AnimBtn.SetAnimation(FlashResRef, 0, A_ANI_PLAYONCE | A_ANI_BLEND)
		AnimBtn.OnAnimEnd(lambda: UpdateMageWindow(Window))
		UpdateMageWindow(Window)

	index = btn.Value
	if GemRB.MemorizeSpell (pc, spelltype, level, index):
		GemRB.PlaySound ("GAM_24")
		Button = MageWindow.GetControl(index + 27)
		Button.SetAnimation (FlashResRef, 0, A_ANI_PLAYONCE | A_ANI_BLEND)
		Button.OnAnimEnd(Complete)

	return

def OpenMageSpellRemoveWindow (parentWin, spellIndex):
	if GameCheck.IsBG2OrEE ():
		Window = GemRB.LoadWindow (101, "GUIMG")
	else:
		Window = GemRB.LoadWindow (5, "GUIMG")

	# "Are you sure you want to ....?"
	TextArea = Window.GetControl (3)
	TextArea.SetText (63745)

	# Remove
	Button = Window.GetControl (0)
	Button.SetText (17507)
	
	def RemoveSpell (spellIndex):
		OnMageRemoveSpell (spellIndex)
		Window.Close()
		parentWin.Close()
	
	Button.OnPress (lambda: RemoveSpell(spellIndex))
	Button.MakeDefault()

	# Cancel
	Button = Window.GetControl (1)
	Button.SetText (13727)
	Button.OnPress (Window.Close)
	Button.MakeEscape()

	Window.ShowModal (MODAL_SHADOW_GRAY)
	return

def OpenMageSpellUnmemorizeWindow (btn):
	if GameCheck.IsBG2OrEE ():
		Window = GemRB.LoadWindow (101, "GUIMG")
	else:
		Window = GemRB.LoadWindow (5, "GUIMG")

	# "Are you sure you want to ....?"
	TextArea = Window.GetControl (3)
	TextArea.SetText (11824)

	# Remove
	Button = Window.GetControl (0)
	Button.SetText (17507)
	
	def Unmemorize (btn):
		OnMageUnmemorizeSpell (btn)
		Window.Close()

	Button.SetValue (btn.Value)
	Button.OnPress (Unmemorize)
	Button.MakeDefault()

	# Cancel
	Button = Window.GetControl (1)
	Button.SetText (13727)
	Button.OnPress (Window.Close)
	Button.MakeEscape()

	Window.ShowModal (MODAL_SHADOW_GRAY)
	return

def OnMageUnmemorizeSpell (btn):
	pc = GemRB.GameGetSelectedPCSingle ()
	level = MageSpellLevel
	spelltype = IE_SPELL_TYPE_WIZARD
	index = btn.Value

	if GemRB.UnmemorizeSpell (pc, spelltype, level, index):
		GemRB.PlaySound ("GAM_44")
		Button = MageWindow.GetControl(index + 3)
		Button.SetAnimation (FlashResRef, 0, A_ANI_PLAYONCE | A_ANI_BLEND)
		Button.OnAnimEnd(lambda: UpdateMageWindow (MageWindow))
	return

def OnMageRemoveSpell (spellIndex):
	pc = GemRB.GameGetSelectedPCSingle ()
	level = MageSpellLevel
	spelltype = IE_SPELL_TYPE_WIZARD

	#remove spell from book
	GemRB.RemoveSpell (pc, spelltype, level, spellIndex)
	UpdateMageWindow (MageWindow)
	return

def LoadCondition ():
	global ContCond, ContTarg

	Table = GemRB.LoadTable ("contcond")
	CondCount = Table.GetRowCount ()
	ContCond = [0] * CondCount

	for i in range (CondCount):
		#get the condition's name and description
		CondTuple = (Table.GetValue (i, 0), Table.GetValue (i, 1))
		ContCond[i] = CondTuple

	Table = GemRB.LoadTable ("conttarg")
	TargCount = Table.GetRowCount ()
	ContTarg = [0] * TargCount

	for i in range (TargCount):
		#get the condition's name and description
		CondTuple = (Table.GetValue (i, 0), Table.GetValue (i, 1))
		ContTarg[i] = CondTuple
	return

def OpenSequencerWindow ():
	global OtherWindow
	global ContingencyTextArea, TypeButton, OkButton
	global Level, MaxLevel, Target, Count
	global pc
	global ContCond, ContTarg
	global Spell1, Spell2, Spell3, Source

	if ContCond == None:
		LoadCondition()

	Level = 1
	Spell1 = ""
	Spell2 = ""
	Spell3 = ""
	#the target player (who receives the contingency or sequencer)
	pc = GemRB.GetVar("P0")
	ClassName = GUICommon.GetClassRowName (pc)
	#maximum spell level
	MaxLevel = GemRB.GetVar("P1")
	#target 0 - any, 1 - caster only, 2 - sequencer
	#spell count 1-3
	p2 = GemRB.GetVar("P2")
	Source = GemRB.GetSpellCastOn(pc)

	print("Source: ", Source)
	Target = p2>>16
	Count = p2&255
	if Count > 3:
		Count = 3

	#saving the original portrait window
	OtherWindow = Window = GemRB.LoadWindow (6, "GUIMG")

	Title = Window.GetControl (0x0fffffff)

	ContingencyTextArea = Window.GetControl (25)

	CondSelect = Window.GetControl (4)
	CondLabel = Window.GetControl (0x10000000)
	TargSelect = Window.GetControl (6)
	TargLabel = Window.GetControl (0x10000001)
	TypeButton = Window.GetControl (8)

	#no cleric spells available
	if CommonTables.ClassSkills.GetValue (ClassName, "DRUIDSPELL") == "*" and\
			CommonTables.ClassSkills.GetValue (ClassName, "CLERICSPELL") == "*":
		TypeButton.SetState (IE_GUI_BUTTON_DISABLED)

	if Target == 2:
		CondSelect.SetVisible (False)
		CondLabel.SetVisible (False)
		TargSelect.SetVisible (False)
		TargLabel.SetVisible (False)
	else:
		CondSelect.OnSelect (ContingencyHelpCondition)
		CondSelect.SetOptions ([elem[0] for elem in ContCond], "ContCond", 0)

		TargSelect.OnSelect (ContingencyHelpTarget)
		if Target:
			TargSelect.SetOptions ([ContTarg[0][0]], "ContTarg", 1)
		else:
			TargSelect.SetOptions ([elem[0] for elem in ContTarg], "ContTarg", 0)

	GemRB.SetVar ("SpellType", 0)
	TypeButton.SetVarAssoc ("SpellType", 0)
	TypeButton.SetFlags (IE_GUI_BUTTON_CHECKBOX, OP_OR)
	TypeButton.OnPress (ContTypePressed)

	Button = Window.GetControl (9)
	Button.OnPress (LevelIncrease)
	Button = Window.GetControl (10)
	Button.OnPress (LevelDecrease)

	OkButton = Window.GetControl (27)
	OkButton.SetText (11973)
	OkButton.MakeDefault()
	OkButton.SetState (IE_GUI_BUTTON_DISABLED)

	CancelButton = Window.GetControl (29)
	CancelButton.SetText (13727)
	CancelButton.MakeEscape()

	OkButton.OnPress (ContingencyOk)
	CancelButton.OnPress (ContingencyCancel)
	ContTypePressed ()
	Window.ShowModal (MODAL_SHADOW_GRAY)

	# this is here because core runs the selection change event handler too often
	if Target == 2:
		if Count < 3:
			Title.SetText (55374)
			ContingencyTextArea.SetText (60420)
		else:
			Title.SetText (55375)
			ContingencyTextArea.SetText (55372)
	else:
		if Count < 3:
			Title.SetText (11941)
		else:
			Title.SetText (55376)
		ContingencyTextArea.SetText (55373)

	return

def UpdateSpellList ():
	Window = OtherWindow

	if SpellType:
		Type = IE_SPELL_TYPE_PRIEST
	else:
		Type = IE_SPELL_TYPE_WIZARD

	Label = Window.GetControl (0x1000001d)
	GemRB.SetToken ('LEVEL', str(Level))
	Label.SetText (12137)

	BuildSpellList(pc, Type, Level-1)

	names = list(SpellList.keys())
	names.sort()

	cnt = len(names)
	j = 0
	for i in range(11, 22):
		Button = Window.GetControl (i)
		Button.SetFont ("NUMBER")
		Button.SetFlags (IE_GUI_BUTTON_ALIGN_RIGHT|IE_GUI_BUTTON_ALIGN_BOTTOM,OP_OR)
		if j<cnt:
			Button.SetSpellIcon (names[j], 1)
			Button.SetText( str(SpellList[names[j]]) )
			Button.SetVarAssoc("PickedSpell", j)
			Button.OnPress (ContingencyHelpSpell)
			Button.SetState (IE_GUI_BUTTON_ENABLED)
		else:
			Button.SetSpellIcon("")
			Button.SetText("")
			Button.SetState (IE_GUI_BUTTON_DISABLED)
		j = j+1

	Button = Window.GetControl (22)
	Button.SetSpellIcon(Spell1, 1)
	Button.OnPress (DeleteSpell1)
	Button = Window.GetControl (23)
	Button.SetSpellIcon(Spell2, 1)
	Button.OnPress (DeleteSpell2)
	Button = Window.GetControl (24)
	Button.SetSpellIcon(Spell3, 1)
	Button.OnPress (DeleteSpell3)

	if not Spell1:
		OkButton.SetState (IE_GUI_BUTTON_DISABLED)
	else:
		OkButton.SetState (IE_GUI_BUTTON_ENABLED)
	return

def BuildSpellList (pc, spelltype, level):
	global SpellList

	import Spellbook

	if not Exclusions:
		LoadExclusions()

	SpellList = {}
	memoedSpells = Spellbook.GetMemorizedSpells (pc, spelltype, level)

	# are we a sorcerer-style?
	SorcererBook = Spellbook.HasSorcererBook (pc)

	names = []
	for s in memoedSpells:
		names.append(s["SpellResRef"])

	if SorcererBook:
		dec = 0
		for selected in Spell1, Spell2, Spell3:
			if selected in names:
				dec += 1
		# decrease count for all spells
		if dec > 0:
			for ms in memoedSpells:
				ms["MemoCount"] -= dec
	else:
		for s in memoedSpells:
			sref = s["SpellResRef"]
			for selected in Spell1, Spell2, Spell3:
				if sref == selected:
					s["MemoCount"] -= 1

	for s in memoedSpells:
		if s["MemoCount"] < 1:
			continue

		spell = s["SpellResRef"]
		if spell in Exclusions[level]:
			continue
		SpellList[spell] = s["MemoCount"]
	return

def ContTypePressed ():
	global SpellType

	Label = OtherWindow.GetControl (0x10000002)
	SpellType = GemRB.GetVar ("SpellType")

	if SpellType:
		Label.SetText (54352)
		TypeButton.SetText (18039)
	else:
		Label.SetText (21836)
		TypeButton.SetText (7204)
	UpdateSpellList()
	return

def ContingencyOk ():
	global OtherWindow

	#set the storage
	if Target == 2:
		GemRB.ApplyEffect (pc, "Sequencer:Store", 0, 0, Spell1, Spell2, Spell3, Source)
	else:
		GemRB.ApplyEffect (pc, "CastSpellOnCondition", GemRB.GetVar ("ContTarg"), GemRB.GetVar ("ContCond"), Spell1, Spell2, Spell3, Source)
	#set the innate
	if GemRB.LearnSpell (pc, Source+"d", LS_MEMO):
		GemRB.Log (LOG_ERROR, "ContingencyOk", "Failed to learn sequencer/contingency!")
	OtherWindow.Close ()
	return

def ContingencyCancel ():
	global OtherWindow

	GemRB.SetPlayerStat (pc, IE_IDENTIFYMODE, 0)
	OtherWindow.Close ()
	return

def ContingencyHelpSpell ():
	global Spell1, Spell2, Spell3

	names = list(SpellList.keys())
	names.sort()
	i = GemRB.GetVar("PickedSpell")
	spell = names[i]

	if Spell1=="" and Count>0:
		Spell1 = spell
	elif Spell2=="" and Count>1:
		Spell2 = spell
	elif Spell3=="" and Count>2:
		Spell3 = spell

	spl = GemRB.GetSpell(spell)
	ContingencyTextArea.SetText (spl["SpellDesc"])
	UpdateSpellList ()
	return

def DeleteSpell1 ():
	global Spell1, Spell2, Spell3

	Spell1 = Spell2
	Spell2 = Spell3
	Spell3 = ""
	ContingencyTextArea.SetText("")
	UpdateSpellList ()
	return

def DeleteSpell2 ():
	global Spell2, Spell3

	Spell2 = Spell3
	Spell3 = ""
	ContingencyTextArea.SetText("")
	UpdateSpellList ()
	return

def DeleteSpell3 ():
	global Spell3

	Spell3 = ""
	ContingencyTextArea.SetText("")
	UpdateSpellList ()
	return

def ContingencyHelpCondition ():
	i = GemRB.GetVar ("ContCond")
	ContingencyTextArea.SetText (ContCond[i][1])
	return

def ContingencyHelpTarget ():
	i = GemRB.GetVar ("ContTarg")
	ContingencyTextArea.SetText (ContTarg[i][1])
	return

def LoadExclusions():
	global Exclusions

	ExclusionTable = GemRB.LoadTable ("contingx")
	Columns = ExclusionTable.GetColumnCount ()
	Rows = ExclusionTable.GetRowCount ()
	Exclusions = []
	for i in range (Columns):
		Exclusions.append ([])
		for j in range (Rows):
			spell = ExclusionTable.GetValue (j, i, GTV_STR)
			if spell[0]=="*":
				break
			Exclusions[i].append (spell.lower())

	return

def LevelIncrease():
	global Level

	if Level<MaxLevel:
		Level = Level+1
	UpdateSpellList()
	return

def LevelDecrease():
	global Level

	if Level>1:
		Level = Level-1
	UpdateSpellList()
	return

def OpenContingenciesWindow():
	global OtherWindow

	OtherWindow = Window = GemRB.LoadWindow (7, "GUIMG")

	OkButton = Window.GetControl (2)
	OkButton.SetText (11973)
	OkButton.OnPress (Window.Close)
	OkButton.MakeDefault ()

	pc = GemRB.GameGetSelectedPCSingle ()
	contingencies = GemRB.GetEffects (pc, "CastSpellOnCondition")
	sequencers = GemRB.GetEffects (pc, "Sequencer:Store")
	targets = [ 30992, 30990, 30994, "Anyone"]
	conditions = [ "Was hit", 30973, 30975, 30982, 30984, 30986, 30988, "Attacked", "Near 4'", "Near 10'", "Every round", "Took damage", "Killed", "Time of day", "In personal space", "State", "I died", "Someone died", "Got turned", "HP <", "HP% <", "Spell state" ]
	for cidx in range(5):
		rowID1 = Window.GetControl (0x10000000 + 31 + cidx)
		rowID2 = Window.GetControl (0x10000000 + 37 + cidx)
		rowID3 = Window.GetControl (0x10000000 + 43 + cidx)
		spellBtn1 = Window.GetControl (25 + cidx)
		spellBtn2 = Window.GetControl (31 + cidx)
		spellBtn3 = Window.GetControl (37 + cidx)
		handBtn = Window.GetControl (43 + cidx)
		spellBtn1.SetFlags (IE_GUI_BUTTON_NO_IMAGE, OP_OR)
		spellBtn2.SetFlags (IE_GUI_BUTTON_NO_IMAGE, OP_OR)
		spellBtn3.SetFlags (IE_GUI_BUTTON_NO_IMAGE, OP_OR)
		spellBtn1.SetState (IE_GUI_BUTTON_LOCKED)
		spellBtn2.SetState (IE_GUI_BUTTON_LOCKED)
		spellBtn3.SetState (IE_GUI_BUTTON_LOCKED)
		if cidx < len(contingencies):
			rowID1.SetText (str(cidx + 1))
			cont = contingencies[cidx]
			rowID2.SetText (conditions[cont["Param2"]])
			rowID3.SetText (targets[cont["Param1"]])

			spellBtn1.SetBAM (cont["Spell1Icon"][:-1] + "b", 0, 0)
			if cont["Resource2"] != "":
				spellBtn2.SetBAM (cont["Spell2Icon"][:-1] + "b", 0, 0)
			if cont["Resource3"] != "":
				spellBtn3.SetBAM (cont["Spell3Icon"][:-1] + "b", 0, 0)

			# warning: can take out too many
			handBtn.OnPress (lambda: GemRB.DispelEffect (pc, "CastSpellOnCondition", cont["Param2"]))
		elif cidx < len(contingencies) + len(sequencers):
			rowID1.SetText (str(cidx + 1))
			cont = sequencers[cidx - len(contingencies)]
			rowID2.SetText (25951)
			rowID3.SetText (None)

			spellBtn1.SetBAM (cont["Spell1Icon"][:-1] + "b", 0, 0)
			if cont["Resource2"] != "":
				spellBtn2.SetBAM (cont["Spell2Icon"][:-1] + "b", 0, 0)
			if cont["Resource3"] != "":
				spellBtn3.SetBAM (cont["Spell3Icon"][:-1] + "b", 0, 0)

			# warning: can take out too many
			handBtn.OnPress (lambda: GemRB.DispelEffect (pc, "CastSpellOnCondition", cont["Param2"]))
		else:
			rowID1.SetText (None)
			rowID2.SetText (None)
			rowID3.SetText (None)
			spellBtn1.SetFlags (IE_GUI_BUTTON_NO_IMAGE, OP_SET)
			spellBtn1.SetState (IE_GUI_BUTTON_DISABLED)
			spellBtn2.SetFlags (IE_GUI_BUTTON_NO_IMAGE, OP_SET)
			spellBtn2.SetState (IE_GUI_BUTTON_DISABLED)
			spellBtn3.SetFlags (IE_GUI_BUTTON_NO_IMAGE, OP_SET)
			spellBtn3.SetState (IE_GUI_BUTTON_DISABLED)
			handBtn.SetFlags (IE_GUI_BUTTON_NO_IMAGE, OP_OR)


###################################################
# End of file GUIMG.py



# GemRB - Infinity Engine Emulator
# Copyright (C) 2003 The GemRB Project
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
#


# GUIWORLD.py - scripts to control some windows from GUIWORLD winpack
#    except of Actions, Portrait, Options and Dialog windows

###################################################

import GemRB
import GUICommon
import GUICommonWindows
import CommonWindow
import Container
from GUIDefines import *
import MessageWindow

FormationWindow = None

def DialogStarted ():
	global ContinueWindow

	GUICommonWindows.CloseTopWindow()
	Container.CloseContainerWindow()

	# opening control size to maximum, enabling dialog window
	CommonWindow.SetGameGUIHidden(False)
	GemRB.GameSetScreenFlags(GS_DIALOG, OP_OR)

	MWin = GemRB.GetView("MSGWIN")
	CloseButton= MWin.GetControl (0)
	CloseButton.SetText ("")
	CloseButton.SetDisabled(True)
	CloseButton.SetFlags (IE_GUI_BUTTON_NO_IMAGE, OP_OR)

def DialogEnded ():
	Button = MessageWindow.MWindow.GetControl (0)
	Button.MakeDefault(True)
	Button.SetDisabled(False)
	Button.SetText (28082)
	Button.SetFlags (IE_GUI_BUTTON_NO_IMAGE, OP_NAND)

def CloseContinueWindow ():
	# don't close the actual window now to avoid flickering: we might still want it open
	GemRB.SetVar ("DialogChoose", GemRB.GetVar ("DialogOption"))

def NextDialogState ():
	if not MessageWindow.MWindow:
		return

	Button = MessageWindow.MWindow.GetControl (0)
	Button.OnPress (None)

def OpenEndMessageWindow ():
	Button = MessageWindow.MWindow.GetControl (0)
	Button.SetText (34602)
	Button.OnPress (CloseContinueWindow)
	Button.MakeDefault(True)
	Button.SetDisabled(False)

def OpenContinueMessageWindow ():
	#continue
	Button = MessageWindow.MWindow.GetControl (0)
	Button.SetText (34603)
	Button.OnPress (CloseContinueWindow)
	Button.MakeDefault(True)
	Button.SetDisabled(False)

last_formation = None

def OpenFormationWindow ():
	global FormationWindow

	FormationWindow = Window = GemRB.LoadWindow (27, GUICommon.GetWindowPack(), WINDOW_BOTTOM|WINDOW_HCENTER)
	FormationWindow.SetFlags (IE_GUI_VIEW_IGNORE_EVENTS, OP_OR)

	# Done
	Button = Window.GetControl (13)
	Button.SetText (1403)
	Button.OnPress (FormationWindow.Close)

	for i in range (13):
		Button = Window.GetControl (i)
		Button.SetVarAssoc ("SelectedFormation", i)
		Button.SetTooltip (48152 if i == 6 else 44957 + i)
		Button.OnPress (SelectFormation)

	GemRB.SetVar ("SelectedFormation", GemRB.GameGetFormation (0))
	SelectFormation ()

def SelectFormation ():
	global last_formation
	Window = FormationWindow
	
	formation = GemRB.GetVar ("SelectedFormation")
	print("FORMATION:", formation)
	if last_formation != None and last_formation != formation:
		Button = Window.GetControl (last_formation)
		Button.SetState (IE_GUI_BUTTON_UNPRESSED)

	Button = Window.GetControl (formation)
	Button.SetState (IE_GUI_BUTTON_SELECTED)

	last_formation = formation



# GemRB - Infinity Engine Emulator
# Copyright (C) 2003-2005 The GemRB Project
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
#

# MessageWindow.py - scripts and GUI for the main (walk) window

###################################################

import GemRB
import ActionsWindow as ActionsWindowModule
import GameCheck
import GUICommon
import GUICommonWindows
import PortraitWindow

from GUIDefines import *

def OnLoad():
	# just load the medium window always. we can shrink/expand it, but it is the one with both controls
	# this saves us from having to bend over backwards to load the new window and move the text to it (its also shorter code)
	# for reference: medium = 12 = guiwdmb8, large = 7 = guwbtp38, small = 4 = guwbtp28
	MessageWindow = GemRB.LoadWindow(12, GUICommon.GetWindowPack(), WINDOW_BOTTOM|WINDOW_HCENTER)
	MessageWindow.SetFlags(WF_BORDERLESS|IE_GUI_VIEW_IGNORE_EVENTS, OP_OR)
	MessageWindow.AddAlias("MSGWIN")
	MessageWindow.AddAlias("HIDE_CUT", 0)
	
	TMessageTA = MessageWindow.GetControl(1)
	TMessageTA.SetFlags(IE_GUI_TEXTAREA_AUTOSCROLL|IE_GUI_TEXTAREA_HISTORY)
	TMessageTA.SetResizeFlags(IE_GUI_VIEW_RESIZE_ALL)
	TMessageTA.AddAlias("MsgSys", 0)
	TMessageTA.SetColor(ColorRed, TA_COLOR_OPTIONS)
	TMessageTA.SetColor(ColorWhite, TA_COLOR_HOVER)
	
	sbar = MessageWindow.GetControl(2)
	sbar.SetResizeFlags(IE_GUI_VIEW_RESIZE_VERTICAL)

	ActionsWindow = GemRB.LoadWindow(3, GUICommon.GetWindowPack(), WINDOW_BOTTOM|WINDOW_HCENTER)
	ActionsWindowModule.OpenActionsWindowControls (ActionsWindow)
	ActionsWindow.SetFlags(WF_BORDERLESS|IE_GUI_VIEW_IGNORE_EVENTS, OP_OR)
	ActionsWindow.AddAlias("HIDE_CUT", 1)
	ActionsWindow.AddAlias("NOT_DLG", 0)
	
	aFrame = ActionsWindow.GetFrame()
	mFrame = MessageWindow.GetFrame()
	MessageWindow.SetPos(mFrame['x'], mFrame['y'] - aFrame['h'])
	
	if GameCheck.HasHOW():
		OptionsWindow = GemRB.LoadWindow(25, GUICommon.GetWindowPack(), WINDOW_LEFT|WINDOW_VCENTER)
	else:
		OptionsWindow = GemRB.LoadWindow(0, GUICommon.GetWindowPack(), WINDOW_LEFT|WINDOW_VCENTER)
	OptionsWindow.SetFlags(WF_BORDERLESS|IE_GUI_VIEW_IGNORE_EVENTS, OP_OR)
	OptionsWindow.AddAlias("OPTWIN")
	OptionsWindow.AddAlias("HIDE_CUT", 2)
	OptionsWindow.AddAlias("NOT_DLG", 1)
	
	GUICommonWindows.SetupMenuWindowControls (OptionsWindow, 1, None)
	PortraitWin = PortraitWindow.OpenPortraitWindow(1)
	PortraitWin.AddAlias("HIDE_CUT", 3)
	PortraitWin.AddAlias("NOT_DLG", 2)

	Button = ActionsWindow.GetControl (60)
	if Button:
		Button.OnPress (lambda: ToggleWindowMinimize(OptionsWindow, GS_OPTIONPANE))
		Button = ActionsWindow.GetControl (61)
		Button.OnPress (lambda: ToggleWindowMinimize(PortraitWin, GS_PORTRAITPANE))

	# 1280 and higher don't have this control
	Button = OptionsWindow.GetControl (10)
	if Button:
		Button.OnPress (lambda: ToggleWindowMinimize(OptionsWindow, GS_OPTIONPANE))
		Button = PortraitWin.GetControl (8)
		Button.OnPress (lambda: ToggleWindowMinimize(PortraitWin, GS_PORTRAITPANE))

	UpdateControlStatus(True)

# or if we are going to do this a lot maybe add a view flag for automatically resizing to the assigned background
# TODO: add a GUIScript function to return a dict with a CObject<Sprite2D> + dimensions for a given resref
WinSizes = {GS_SMALLDIALOG : 45,
			GS_MEDIUMDIALOG : 109,
			GS_LARGEDIALOG : 237}

def MWinBG(size, width=None):
	if width is None:
		width = GemRB.GetSystemVariable (SV_WIDTH)

	bg = None
	if size == GS_SMALLDIALOG:
		if width == 640:
			bg = "guiwbtp2"
		else:
			bg = "guwbtp2"
	elif size == GS_MEDIUMDIALOG:
		bg = "guiwdmb"
	elif size == GS_LARGEDIALOG:
		if width == 640:
			bg = "guiwbtp3"
		else:
			bg = "guwbtp3"
	else:
		raise ValueError('Invalid size for MWinBG')

	if width >= 800 and width < 1024:
		bg = bg + "8"
	elif width >= 1024:
		bg = bg + "0"

	# FIXME: infinite recursion possible
	from ie_restype import RES_MOS
	if not GemRB.HasResource(bg, RES_MOS):
		if GameCheck.IsBG2OrEE () or GameCheck.IsIWD2():
			return MWinBG(size, 800)
		else:
			return MWinBG(size, 640)

	return bg

MinimizedWindows = {}
def ToggleWindowMinimize(win, GSFlag = 0):
	key = win.ID
	maximize = True if key in MinimizedWindows else False
	if maximize:
		# restore to original size
		win.SetSize(*MinimizedWindows[key])
		del MinimizedWindows[key]

		if GSFlag: # stored in save
			GemRB.GameSetScreenFlags(GSFlag, OP_NAND)

	else: # minimize
		size = win.GetSize()
		MinimizedWindows[key] = size
		win.SetSize(size[0], 5)
		win.ScrollTo(0,0)

		if GSFlag: # stored in save
			GemRB.GameSetScreenFlags(GSFlag, OP_OR)
			
	if key == GemRB.GetView("OPTWIN").ID:
		ToggleActionbarClock(not maximize)

def ToggleActionbarClock(show):
	actwin = GemRB.GetView("ACTWIN")
	clock = actwin.GetControl (62)
	if clock:
		clock.SetVisible(show)

MTARestoreSize = None
def UpdateControlStatus(init = False):
	MessageWindow = GemRB.GetView("MSGWIN")

	if not MessageWindow:
		# exit if we get called from core without the right window pack being loaded
		return

	ExpandButton = MessageWindow.GetControl(0)
	ExpandButton.SetDisabled(False)
	ExpandButton.SetResizeFlags(IE_GUI_VIEW_RESIZE_TOP)
	ExpandButton.SetHotKey(chr(0x8d), 0, True) # GEM_PGUP

	ContractButton = MessageWindow.GetControl(3)
	ContractButton.SetFlags(IE_GUI_VIEW_INVISIBLE|IE_GUI_VIEW_DISABLED, OP_NAND)
	ContractButton.SetResizeFlags(IE_GUI_VIEW_RESIZE_BOTTOM)
	ContractButton.SetHotKey(chr(0x8e), 0, True) # GEM_PGDOWN
	
	def GetGSFlags():
		GSFlags = GemRB.GetGUIFlags()
		Expand = GSFlags&GS_DIALOGMASK
		GSFlags = GSFlags-Expand
		return (GSFlags, Expand)

	def SetMWSize(size, GSFlags):
		if size not in WinSizes:
			return
		
		frame = ContractButton.GetFrame()
		if size != GS_SMALLDIALOG:
			frame['y'] -= (frame['h'] + 6)
		ExpandButton.SetFrame(frame)

		frame = MessageWindow.GetFrame()
		diff = frame['h'] - WinSizes[size]
		frame['y'] += diff
		frame['h'] = WinSizes[size]
		MessageWindow.SetFrame(frame)
		MessageWindow.SetBackground(MWinBG(size))

		GemRB.GameSetScreenFlags(size + GSFlags, OP_SET)

	def OnIncreaseSize():
		GSFlags, Expand = GetGSFlags()
		Expand = (Expand + 1)*2 # next size up

		SetMWSize(Expand, GSFlags)

	def OnDecreaseSize():
		GSFlags, Expand = GetGSFlags()
		Expand = Expand // 2 - 1 # next size down: 6->2, 2->0

		SetMWSize(Expand, GSFlags)

	global MTARestoreSize
	ContractButton.OnPress (OnDecreaseSize)
	ExpandButton.OnPress (OnIncreaseSize)

	GSFlags, Expand = GetGSFlags()
	if init:
		SetMWSize(Expand, GSFlags)
		if GSFlags&GS_OPTIONPANE:
			win = GemRB.GetView("OPTWIN")
			ToggleWindowMinimize(win)
		else:
			ToggleActionbarClock(False)
		if GSFlags&GS_PORTRAITPANE:
			win = GemRB.GetView("PORTWIN")
			ToggleWindowMinimize(win)
	else:
		if Expand == GS_LARGEDIALOG:
			if MTARestoreSize is not None and (GSFlags&GS_DIALOG) == 0:
				SetMWSize(MTARestoreSize, GSFlags)
				MTARestoreSize = None
			else:
				ExpandButton.SetDisabled(True)

		elif (GSFlags&GS_DIALOG):
			MTARestoreSize = Expand
			#a dialogue is running, setting messagewindow size to maximum
			SetMWSize(GS_LARGEDIALOG, GSFlags)

		elif Expand == GS_SMALLDIALOG:
			ContractButton.SetFlags(IE_GUI_VIEW_INVISIBLE|IE_GUI_VIEW_DISABLED, OP_OR)

	return



import GemRB
import Tests
import GUICommon
import GUICommonWindows
import CommonWindow
import GUIClasses
import GUIINV
import GUIMA
from GUIDefines import *

def OnLoad():
	MessageWindow = GemRB.LoadWindow(0, GUICommon.GetWindowPack(), WINDOW_BOTTOM|WINDOW_HCENTER)
	MessageWindow.SetFlags (WF_BORDERLESS|IE_GUI_VIEW_IGNORE_EVENTS, OP_OR)
	MessageWindow.AddAlias("MSGWIN")
	MessageWindow.AddAlias("HIDE_CUT", 0)

	# set up some *initial* text (UpdateControlStatus will get called several times)
	TMessageTA = MessageWindow.GetControl(0)
	TMessageTA.SetFlags (IE_GUI_TEXTAREA_AUTOSCROLL|IE_GUI_TEXTAREA_HISTORY)
	TMessageTA.SetResizeFlags(IE_GUI_VIEW_RESIZE_ALL)
	TMessageTA.AddAlias("MsgSys", 0)
	TMessageTA.AddAlias("MTA", 0)
	TMessageTA.SetColor(ColorRed, TA_COLOR_OPTIONS)
	TMessageTA.SetColor(ColorWhite, TA_COLOR_HOVER)
	
	results = Tests.RunTests ()
	TMessageTA.SetText ("[cap]D[/cap]emo " + "DEMO "*40 + "\n" + results)

	PauseButton = MessageWindow.GetControl (2)
	PauseButton.OnPress (lambda: GemRB.GamePause (2, 0))
	PauseButton.SetAnimation ("loading", 0, A_ANI_GAMEANIM)
	PauseButton.SetFlags (IE_GUI_BUTTON_PICTURE|IE_GUI_BUTTON_NORMAL, OP_SET)

	MapButton = MessageWindow.GetControl (3)
	MapButton.SetText ("M")
	MapButton.OnPress (GUIMA.OpenMapWindow)
	MapButton.OnRightPress (lambda: GemRB.MoveToArea ("ar0110"))

	CenterButton = MessageWindow.GetControl (4)
	CenterButton.SetText ("C")
	CenterButton.OnPress (lambda: GemRB.GameControlSetScreenFlags (SF_CENTERONACTOR, OP_OR))

	InventoryButton = MessageWindow.GetControl (5)
	InventoryButton.SetText ("I")
	InventoryButton.OnPress (GUIINV.OpenInventoryWindow)

def UpdateControlStatus():
	pass



# GemRB - Infinity Engine Emulator
# Copyright (C) 2007 The GemRB Project
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
#

# DemoEnd.py - display DemoEnd pictures

###################################################

import GemRB

Picture = None
Table = None
Window = None

def OnLoad ():
	global Table, Picture, Window
	
	Window = GemRB.LoadWindow(0, "demoend")
	Picture = 0
	Table = GemRB.LoadTable ("splashsc")
	resref = Table.GetValue (Picture,0)
	Button = Window.GetControl (0)
	Button.SetFlags (IE_GUI_BUTTON_NO_IMAGE|IE_GUI_BUTTON_PICTURE, OP_SET)
	Button.MakeDefault()
	Button.SetState (IE_GUI_BUTTON_LOCKED)
	Button.OnPress (NextPress)
	Button.SetPicture (resref)
	Window.Focus()
	return
	
def NextPress ():
	global Picture
	
	Picture = Picture + 1
	if Table.GetRowCount()<=Picture:
		DemoEnd()
	else:
		resref = Table.GetValue (Picture,0)
		Button = Window.GetControl (0)
		Button.SetPicture (resref)
	return
	
def DemoEnd ():
	GemRB.SetNextScript ("Start")
	return



# GemRB - Infinity Engine Emulator
# Copyright (C) 2003-2004 The GemRB Project
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
#


# GUISPL.py - scripts to control spells windows from GUISPL winpack

###################################################

import GemRB
import GUICommon
import GUICommonWindows
import Spellbook
from GUIDefines import *
from ie_stats import *
from ie_action import ACT_CAST

BookNames = (1083,1079,1080,1078,1077,32,1081,39722)
CurrentPC = None

def InitSpellBookWindow (Window):
	#setup level buttons
	# looping backwards so the selected button gets drawn properly
	for i in range (8, -1, -1):
		Button = Window.GetControl (55 + i)
		Button.OnPress (lambda: UpdateSpellBookWindow(Window))
		Button.SetFlags (IE_GUI_BUTTON_RADIOBUTTON, OP_OR)
		Button.SetVarAssoc ("SpellBookSpellLevel", i)

	# Setup memorized spells buttons
	for i in range (24):
		Button = Window.GetControl (6 + i)
		color = {'r' : 0, 'g' : 0, 'b' : 0, 'a' : 160}
		Button.SetBorder (0,color,0,1)
		#Button.SetBAM ("SPELFRAM",0,0,0)
		Button.SetFlags (IE_GUI_BUTTON_PICTURE | IE_GUI_BUTTON_NO_IMAGE, OP_OR)
		Button.SetVarAssoc("Memorized", i)

	# Setup book spells buttons
	for i in range (8):
		Button = Window.GetControl (30 + i)
		Button.SetState (IE_GUI_BUTTON_LOCKED)
		Button.OnRightPress (OpenSpellBookSpellInfoWindow)

	# Setup book selection
	for i in range(3, -1, -1):
		Button = Window.GetControl (88 + i)
		Button.SetFlags (IE_GUI_BUTTON_RADIOBUTTON, OP_OR)
		Button.SetVarAssoc ("SelectedBook", i)
		Button.OnPress (ChangeBook)

	# book left / right
	Button = Window.GetControl (92)
	Button.OnPress (SpellBookCycle)
	Button = Window.GetControl (93)
	Button.OnPress (SpellBookCycle)

	# scrollbar for known spells
	ScrollBar = Window.GetControl (54)
	Window.SetEventProxy(ScrollBar)
	ScrollBar.OnChange (lambda: UpdateSpellBookWindow(Window))

	return

def NewSpellBookWindow(Window):
	Window.AddAlias("WIN_SPL")
	InitSpellBookWindow(Window)
	GemRB.SetVar ("SpellBookSpellLevel", 0)

def GetActiveSpellBooks(pc):
	return [i for i in range(8) if GemRB.GetMemorizableSpellsCount(pc, i, 0) > 0]
	
def ChangeBook(btn):
	UpdateSpellBookWindow (btn.Window)

def GetBookType(pc):
	BookIndex = GemRB.GetVar ("SelectedBook")
	ActiveSpellBooks = GetActiveSpellBooks(pc)
	return ActiveSpellBooks[BookIndex]

def SelectedNewPlayer (Window):
	global CurrentPC

	# we call the selection change handler much too often and it breaks scrolling
	pc = GemRB.GameGetSelectedPCSingle ()
	if CurrentPC == pc:
		return
	else:
		CurrentPC = pc

	ScrollBar = Window.GetControl (54)
	ScrollBar.ScrollTo (0, 0)
	UpdateSpellBookWindow (Window)
	return

ToggleSpellBookWindow = GUICommonWindows.CreateTopWinLoader(2, "GUISPL", GUICommonWindows.ToggleWindow, NewSpellBookWindow, SelectedNewPlayer, True)
OpenSpellBookWindow = GUICommonWindows.CreateTopWinLoader(2, "GUISPL", GUICommonWindows.OpenWindowOnce, NewSpellBookWindow, SelectedNewPlayer, True)

def SpellBookCycle (btn):
	Button = btn.Window.GetControl (88 + btn.Value)
	Button.SetState (IE_GUI_BUTTON_SELECTED)
	UpdateSpellBookWindow (btn.Window)
	return

def UpdateSpellBookWindow (Window):
	pc = GemRB.GameGetSelectedPCSingle ()
	ActiveSpellBooks = GetActiveSpellBooks(pc)
	NumBooks = len(ActiveSpellBooks)

	# update spellbook buttons
	ActiveSpellBooks = GetActiveSpellBooks (pc)
	for i in range(3, -1, -1):
		Button = Window.GetControl (88 + i)
		if len(ActiveSpellBooks) > i:
			Button.SetState (IE_GUI_BUTTON_ENABLED)
			BookType = ActiveSpellBooks[i]
			Button.SetText (BookNames[BookType])
		else:
			Button.SetState (IE_GUI_BUTTON_DISABLED)
			Button.SetText ("")

	if len(ActiveSpellBooks) == 0:
		GemRB.SetVar ("SelectedBook", None)
	else:
		Button = Window.GetControl (88)
		Button.SetState (IE_GUI_BUTTON_SELECTED)

	BookIndex = GemRB.GetVar ("SelectedBook")
	if BookIndex is not None:
		Button = Window.GetControl (92)
		Button.SetDisabled(BookIndex <= 0)
		Button.SetValue(BookIndex - 1)

		Button = Window.GetControl (93)
		Button.SetDisabled(BookIndex >= NumBooks - 1)
		Button.SetValue(BookIndex + 1)
		BookType = ActiveSpellBooks[BookIndex]
	else:
		print ("no book selected")
		BookType = 0

	level = GemRB.GetVar("SpellBookSpellLevel")
	sorcerer_style = (BookType == IE_IWD2_SPELL_BARD) or (BookType == IE_IWD2_SPELL_SORCERER)

	Name = GemRB.GetPlayerName (pc, 0)
	Label = Window.GetControl (0xfffffff)
	Label.SetText (Name)

	Button = Window.GetControl (1)
	Button.SetPicture (GemRB.GetPlayerPortrait (pc, 0)["Sprite"])
	Button.SetState (IE_GUI_BUTTON_LOCKED)

	memorized = MemorizedSpellList(pc, BookType, level)
	mem_cnt = len(memorized)

	for i in range (24):
		Button = Window.GetControl (6 + i)
		if GemRB.GetVar("{}_ANIM".format(Button.ControlID)): # we have an animation in progress, bail and it will refresh us after its done
			Button.SetFlags(IE_GUI_VIEW_IGNORE_EVENTS, OP_OR)
			Button.SetVisible(True)
			continue
		else:
			Button.SetFlags(IE_GUI_VIEW_IGNORE_EVENTS, OP_NAND)

		Label = Window.GetControl (0x1000003f+i)
		# actually, it doesn't display any memorized spells for sorcerer-style spellbooks
		if i < mem_cnt and not sorcerer_style:
			ms = memorized[i]
			spell = ms['SpellResRef']
			Button.SetSpellIcon (spell)
			Button.SetTooltip (ms['SpellName'])
			# since spells are stacked, we need to check first whether to unmemorize (deplete) or remove (already depleted)
			if ms['MemoCount'] < ms['KnownCount']:
				# already depleted, just remove
				Button.OnPress (lambda btn, mc = ms['MemoCount'], idx = ms['SpellIndex']: OnSpellBookUnmemorizeSpell(btn, mc, idx))
			else:
				# deplete and remove
				Button.OnPress (lambda btn, idx = ms['SpellIndex']: OpenSpellBookSpellRemoveWindow(btn, idx))
			Button.OnRightPress (lambda btn, idx = ms['SpellIndex']: OpenSpellBookSpellInfoWindow(btn, idx))
			tmp = str(ms['MemoCount'])+"/"+str(ms['KnownCount'])
			Label.SetText (tmp)
			Button.SetVisible(True)
		else:
			Button.SetVisible(False)
			Label.SetText ('')

	known = KnownSpellList(pc, BookType, level)
	known_cnt = len(known)
	SpellTopIndex = GemRB.GetVar ("SpellTopIndex") or 0
	for i in range (8):
		Button = Window.GetControl (30 + i)
		Button.SetFlags (IE_GUI_BUTTON_NO_IMAGE, OP_OR)
		Label = Window.GetControl (0x10000025 + i)
		SpellIdx = i + SpellTopIndex
		if SpellIdx < known_cnt:
			ks = known[SpellIdx]
			spell = ks['SpellResRef']
			Button.SetSpellIcon (spell)
			Button.SetValue(SpellIdx)
			if not sorcerer_style:
				Button.OnPress (OnSpellBookMemorizeSpell)
			Label.SetText (ks['SpellName'])
			Button.SetVisible(True)
		else:
			Button.SetVisible(False)
			Label.SetText("")

	# number of available spell slots
	# sorcerer-style books are different, since max_mem_cnt holds the total level capacity, not the slot count
	# and they display just the current and total number of memorizations per level
	max_mem_cnt = GemRB.GetMemorizableSpellsCount (pc, BookType, level)
	mem_cnt = GetMemorizedSpellsCount (memorized) # count of unique memorized spells
	true_mem_cnt = GemRB.GetMemorizedSpellsCount (pc, BookType, level, True)
	Label = Window.GetControl (0x10000004)
	if sorcerer_style:
		available = "0"
		if known_cnt:
			available = str(true_mem_cnt // known_cnt)
		Label.SetText (available+"/"+str(max_mem_cnt))
	else:
		# reset mem_cnt to take into account stacks
		mem_cnt = GetMemorizedSpellsCount (memorized, True)
		Label.SetText (str(max_mem_cnt-mem_cnt))

	#if actor is uncontrollable, make this grayed
	CantCast = GemRB.GetPlayerStat(pc, IE_DISABLEDBUTTON)&(1<<ACT_CAST)
	GUICommon.AdjustWindowVisibility (Window, pc, CantCast)

	ScrollBar = Window.GetControl (54)
	ScrollBar.SetVarAssoc ("SpellTopIndex", SpellTopIndex, 0, known_cnt - 8)

	return

def GetMemorizedSpellsCount (memorized, total = False):
	'''count the real number of spells in MemorizedSpellList'''
	# can't use len here, since there could be more than one memorization (a "stack")
	# put the counts into a list and sum it, but be careful about depleted spells
	# eg. fully depleted should still count as 1 and each partial too
	count = 'MemoCount' # non-depleted only
	if total:
		count = 'KnownCount' # all

	counts = [max(x[count], 1) for x in memorized]
	return sum(counts)

def MemorizedSpellList (pc, SelectedBook, SpellBookSpellLevel):
	return Spellbook.GetMemorizedSpells (pc, SelectedBook, SpellBookSpellLevel)

def KnownSpellList (pc, SelectedBook, SpellBookSpellLevel):
	return Spellbook.GetKnownSpellsLevel (pc, SelectedBook, SpellBookSpellLevel)

def OpenSpellBookSpellInfoWindow (btn, spellIdx = None):
	Window = GemRB.LoadWindow (3, "GUISPL")

	#back
	Button = Window.GetControl (5)
	Button.SetText (15416)
	Button.MakeEscape()
	Button.OnPress (Window.Close)

	pc = GemRB.GameGetSelectedPCSingle ()
	level = GemRB.GetVar("SpellBookSpellLevel")
	BookType = GetBookType(pc)

	if btn.VarName == "Memorized":
		idx = spellIdx if (spellIdx is not None) else btn.Value
		ms = GemRB.GetMemorizedSpell (pc, BookType, level, idx)
		ResRef = ms['SpellResRef']
	else:
		known = KnownSpellList(pc, BookType, level)
		ResRef = known[btn.Value]["SpellResRef"]
	spell = GemRB.GetSpell (ResRef)

	Label = Window.GetControl (0x0fffffff)
	Label.SetText (spell['SpellName'])

	Button = Window.GetControl (2)
	Button.SetSpellIcon (ResRef)

	Text = Window.GetControl (3)
	Text.SetText (spell['SpellDesc'])

	Window.ShowModal (MODAL_SHADOW_GRAY)
	return

def OnSpellBookMemorizeSpell (btn):
	pc = GemRB.GameGetSelectedPCSingle ()
	level = GemRB.GetVar("SpellBookSpellLevel")
	BookType = GetBookType(pc)
	Window = GemRB.GetView("WIN_SPL")

	spell = GemRB.GetKnownSpell(pc, BookType, level, btn.Value)
	memorized = MemorizedSpellList(pc, BookType, level)
	for idx in range(len(memorized)):
		if memorized[idx]["SpellResRef"] == spell["SpellResRef"]:
			break
	else: # not found, use next slot
		idx = len(memorized)

	def Complete():
		AnimBtn = Window.GetControl(idx + 6)
		AnimBtn.SetAnimation("FLASH", 0, A_ANI_PLAYONCE | A_ANI_BLEND)
		AnimBtn.OnAnimEnd(lambda: UpdateSpellBookWindow (Window))

	if GemRB.MemorizeSpell (pc, BookType, level, btn.Value):
		GemRB.PlaySound ("GAM_24")
		btn.SetAnimation ("FLASH", 0, A_ANI_PLAYONCE | A_ANI_BLEND)
		btn.OnAnimEnd(Complete)
		UpdateSpellBookWindow (Window)

	return

def OpenSpellBookSpellRemoveWindow (btn, spellIdx):
	Window = GemRB.LoadWindow (5, "GUISPL")

	# "Are you sure you want to ....?"
	TextArea = Window.GetControl (3)
	TextArea.SetText (11824)

	# Remove
	Button = Window.GetControl (0)
	Button.SetText (17507)
	Button.SetValue (spellIdx)
	Button.OnPress (OnSpellBookRemoveSpell)
	Button.MakeDefault()

	# Cancel
	Button = Window.GetControl (1)
	Button.SetText (13727)
	Button.OnPress (lambda: Window.Close())
	Button.MakeEscape()

	Window.ShowModal (MODAL_SHADOW_GRAY)
	return

# since we can have semidepleted stacks, first make sure we unmemorize the depleted ones, only then unmemorize
def OnSpellBookUnmemorizeSpell (btn, mem_cnt, spellIdx):
	# remove one depleted spell without touching the still memorized
	# we'd need a different spell index, but it's all handled in the core
	UnmemoSpell(btn, mem_cnt != 0, spellIdx)
	return

# not like removing spells in bg2, where you could delete known spells from the spellbook!
# it's about removing an undepleted memorization
def OnSpellBookRemoveSpell (btn):
	UnmemoSpell (btn)
	btn.Window.Close()
	return

def UnmemoSpell (btn, onlydepleted = False, spellIdx = None):
	pc = GemRB.GameGetSelectedPCSingle ()
	level = GemRB.GetVar("SpellBookSpellLevel")
	BookType = GetBookType(pc)

	Window = GemRB.GetView("WIN_SPL")
	def Complete(btn):
		# remove spell from memory
		idx = spellIdx if (spellIdx is not None) else btn.Value
		GemRB.UnmemorizeSpell (pc, BookType, level, idx, onlydepleted)
		UpdateSpellBookWindow(Window)

	GemRB.PlaySound ("GAM_44")
	AnimBtn = Window.GetControl(6 + btn.Value)
	AnimBtn.SetAnimation ("FLASH", 0, A_ANI_PLAYONCE | A_ANI_BLEND)
	AnimBtn.OnAnimEnd(Complete)

###################################################
# End of file GUISPL.py



# GemRB - Infinity Engine Emulator
# Copyright (C) 2003 The GemRB Project
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
#


# GUIJRNL.py - scripts to control journal/diary windows from GUIJRNL winpack

###################################################
import GemRB
import GUICommon
import GUICommonWindows
from GUIDefines import *

###################################################
JournalWindow = None
Chapter = 0
StartTime = 0
StartYear = 0
###################################################

def InitJournalWindow (Window):
	global StartTime, StartYear
	global JournalWindow, Chapter

	Table = GemRB.LoadTable("YEARS")
	#StartTime is the time offset for ingame time, beginning from the startyear
	StartTime = Table.GetValue("STARTTIME", "VALUE") // 4500
	#StartYear is the year of the lowest ingame date to be printed
	StartYear = Table.GetValue("STARTYEAR", "VALUE")
		
	JournalWindow = Window
	
	Button = Window.GetControl (3)
	Button.OnPress (JournalPrevSectionPress)

	Button = Window.GetControl (4)
	Button.OnPress (JournalNextSectionPress)

	Chapter = GemRB.GetGameVar("chapter")
	UpdateJournalWindow (JournalWindow)
	return

###################################################
def UpdateJournalWindow (Window):
	# Title
	Title = Window.GetControl (5)
	Title.SetText ("[color=FFFF00]" + GemRB.GetString(16202+Chapter) + "[/color]")

	# text area
	Text = Window.GetControl (1)
	Text.SetFlags (IE_GUI_TEXTAREA_AUTOSCROLL)
	Text.Clear ()

	JournalText = ""
	for i in range (GemRB.GetJournalSize (Chapter)):
		je = GemRB.GetJournalEntry (Chapter, i)

		if je == None:
			continue

		hours = je['GameTime'] // 4500
		days = int(hours/24)
		year = str (StartYear + int(days/365))
		dayandmonth = int(StartTime) + days % 365
		GemRB.SetToken("GAMEDAY", str(days) )
		GemRB.SetToken("HOUR",str(hours%24 ) )
		GemRB.SetVar("DAYANDMONTH",dayandmonth)
		GemRB.SetToken("YEAR",year)
		JournalText = JournalText + "[color=FFFF00]" + GemRB.GetString(15980) + "[/color]\n"
		JournalText = JournalText + GemRB.GetString(je['Text']) + "\n\n"
	Text.SetText (JournalText)

ToggleJournalWindow = GUICommonWindows.CreateTopWinLoader(2, "GUIJRNL", GUICommonWindows.ToggleWindow, InitJournalWindow, None, True)
OpenJournalWindow = GUICommonWindows.CreateTopWinLoader(2, "GUIJRNL", GUICommonWindows.OpenWindowOnce, InitJournalWindow, None, True)


###################################################
def JournalPrevSectionPress ():
	global Chapter

	if Chapter > 0:
		Chapter = Chapter - 1
		UpdateJournalWindow (JournalWindow)


###################################################
def JournalNextSectionPress ():
	global Chapter

	#if GemRB.GetJournalSize (Chapter + 1) > 0:
	if Chapter < GemRB.GetGameVar("chapter"):
		Chapter = Chapter + 1
		UpdateJournalWindow (JournalWindow)


###################################################
# End of file GUIJRNL.py



# GemRB - Infinity Engine Emulator
# Copyright (C) 2003-2004 The GemRB Project
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
#

# LoadScreen.py - display Loading screen

###################################################

import GemRB
import GameCheck
from GUIDefines import *

LoadScreen = None

def SetLoadScreen ():
	return

def StartLoadScreen ():
	global LoadScreen

	LoadScreen = GemRB.LoadWindow (0, "guils")
	LoadScreen.AddAlias("LOADWIN")
	
	Middle = LoadScreen.GetControl (3)
	Progress = GemRB.GetVar ("Progress")

	if not GameCheck.IsBG2Demo () and not GameCheck.IsBG2EE ():
		LoadPic = GemRB.GetGameString (STR_LOADMOS)
		if LoadPic == "":
			#the addition of 1 is not an error, bg2 loadpic resrefs are GTRSK002-GTRSK006
			LoadPic = "GTRSK00"+str(GemRB.Roll(1,5,1) )
		Middle.SetPicture (LoadPic)
	elif not GameCheck.IsBG2EE ():
		# During loading, this fn is called at 0% and 70%, so take advantage of that
		#   and display the "quiet" frame first and the "flaming" frame the second time.
		#   It would be even better to display the phases inbetween as well; however,
		#   the bg2demo does not either, even though the frames are there.
		if Progress:
			Middle.SetBAM ("COADCNTR", 1, 0)
		else:
			Middle.SetBAM ("COADCNTR", 0, 0)

	if GameCheck.HasTOB():
		Table = GemRB.LoadTable ("loadh25")
	else:
		Table = GemRB.LoadTable ("loadhint")
	tmp = Table.GetRowCount ()
	tmp = GemRB.Roll (1, tmp-1, 0)
	HintStr = Table.GetValue (tmp, 0)

	Label = LoadScreen.GetControl (2)
	Label.SetText(HintStr)

	def EndLoadScreen ():
		TMessageTA = GemRB.GetView("MsgSys", 0)
		TMessageTA.Append("[p][color=f1f28d]" + GemRB.GetString (HintStr) + "[/color][/p]\n")
		
		if GameCheck.IsBG2Demo():
			Middle = LoadScreen.GetControl (3)
			Middle.SetBAM ("COADCNTR", 1, 0)
	
		LoadScreen.SetAction(lambda win: GemRB.GamePause(0, 0), ACTION_WINDOW_CLOSED)
		GemRB.SetTimer(LoadScreen.Close, 500, 0)
		return

	Bar = LoadScreen.GetControl (0)
	Bar.AddAlias("LOAD_PROG")
	Bar.SetVarAssoc ("Progress", 0)
	Bar.OnEndReached (EndLoadScreen)
	LoadScreen.ShowModal(MODAL_SHADOW_NONE)





# GemRB - Infinity Engine Emulator
# Copyright (C) 2003 The GemRB Project
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
#
#
#character generation, class (GUICG2)
import GemRB
from GUIDefines import *
from ie_stats import *
import CharGenCommon
import GUICommon
import CommonTables


ClassWindow = 0
TextAreaControl = 0
DoneButton = 0

def OnLoad():
	global ClassWindow, TextAreaControl, DoneButton

	MyChar = GemRB.GetVar ("Slot")
	
	GemRB.SetVar("Class",0)
	GemRB.SetVar("Multi Class",0)
	GemRB.SetVar("Specialist",0)
	GemRB.SetVar("Class Kit",0)
	
	ClassCount = CommonTables.Classes.GetRowCount()+1
	ClassWindow = GemRB.LoadWindow(2, "GUICG")
	RaceRow = CommonTables.Races.FindValue(3,GemRB.GetPlayerStat (MyChar, IE_RACE))
	RaceName = CommonTables.Races.GetRowName(RaceRow)

	#radiobutton groups must be set up before doing anything else to them
	for i in range(1,ClassCount):
		ClassName = CommonTables.Classes.GetRowName (i-1)
		if CommonTables.Classes.GetValue(ClassName, "MULTI"):
			continue
			
		Button = ClassWindow.GetControl(i+1)
		Button.SetFlags(IE_GUI_BUTTON_RADIOBUTTON, OP_OR)
		Button.SetState(IE_GUI_BUTTON_DISABLED)

	GemRB.SetVar("MAGESCHOOL",0) 
	HasMulti = 0
	for i in range(1,ClassCount):
		ClassName = CommonTables.Classes.GetRowName(i-1)
		Allowed = CommonTables.Classes.GetValue(ClassName, RaceName)
		if CommonTables.Classes.GetValue (ClassName, "MULTI"):
			if Allowed!=0:
				HasMulti = 1
			continue
			
		Button = ClassWindow.GetControl(i+1)
		
		t = CommonTables.Classes.GetValue(ClassName, "NAME_REF")
		Button.SetText(t )

		if Allowed==2:
			GemRB.SetVar("MAGESCHOOL",5) #illusionist
		if Allowed!=1:
			continue
		Button.SetState(IE_GUI_BUTTON_ENABLED)
		Button.OnPress (ClassPress)
		Button.SetVarAssoc("Class", i)

	MultiClassButton = ClassWindow.GetControl(10)
	MultiClassButton.SetText(11993)
	if HasMulti == 0:
		MultiClassButton.SetState(IE_GUI_BUTTON_DISABLED)

	Allowed = CommonTables.Classes.GetValue ("MAGE", RaceName)
	SpecialistButton = ClassWindow.GetControl(11)
	SpecialistButton.SetText(11994)
	if Allowed == 0:
		SpecialistButton.SetState(IE_GUI_BUTTON_DISABLED)
	
	BackButton = ClassWindow.GetControl(14)
	BackButton.SetText(15416)
	DoneButton = ClassWindow.GetControl(0)
	DoneButton.SetText(11973)
	DoneButton.MakeDefault()

	TextAreaControl = ClassWindow.GetControl(13)

	ClassName = GUICommon.GetClassRowName (GemRB.GetVar ("Class")-1, "index")
	if ClassName == "":
		TextAreaControl.SetText(17242)
		DoneButton.SetState(IE_GUI_BUTTON_DISABLED)
	else:
		TextAreaControl.SetText (CommonTables.Classes.GetValue (ClassName, "DESC_REF"))
		DoneButton.SetState(IE_GUI_BUTTON_ENABLED)

	MultiClassButton.OnPress (MultiClassPress)
	SpecialistButton.OnPress (SpecialistPress)
	DoneButton.OnPress (NextPress)
	BackButton.OnPress (lambda: CharGenCommon.back(ClassWindow))
	ClassWindow.ShowModal(MODAL_SHADOW_GRAY)
	return

def MultiClassPress():
	ClassWindow.Close ()
	GemRB.SetVar("Multi Class",1)
	CharGenCommon.next()

def SpecialistPress():
	ClassWindow.Close ()
	GemRB.SetVar("Specialist",1)

	GemRB.SetVar("Class Kit", 0)
	GemRB.SetVar("Class", 6)
	CharGenCommon.next()
	
def ClassPress():
	ClassName = GUICommon.GetClassRowName (GemRB.GetVar ("Class")-1, "index")
	TextAreaControl.SetText (CommonTables.Classes.GetValue (ClassName, "DESC_REF") )
	DoneButton.SetState(IE_GUI_BUTTON_ENABLED)
	return

def NextPress():
	ClassWindow.Close()
	# find the class from the class table
	ClassName = GUICommon.GetClassRowName (GemRB.GetVar ("Class")-1, "index")
	Class = CommonTables.Classes.GetValue (ClassName, "ID")
	MyChar = GemRB.GetVar ("Slot")
	GemRB.SetPlayerStat (MyChar, IE_CLASS, Class)
	CharGenCommon.next()
	



# GemRB - Infinity Engine Emulator
# Copyright (C) 2003 The GemRB Project
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
#

# Actor.py: Contains the actor class.

import GemRB
import GUICommon
from GUIDefines import *
from ie_stats import *
#this import is primarily for the tables
import CommonTables

##############################################################################
## GLOBALS TO BE INITIALIZED ONCE
##############################################################################
dualswap = None
classcount = None
levelslots = [IE_LEVEL, IE_LEVEL2, IE_LEVEL3]

class Actor:
	"""Holds information of a PC."""

	def __init__ (self, pc):
		"""Load up basic information."""

		#setup our basic Actor
		self.Reset (pc)

		#setup globals if they are blank
		if dualswap == None:
			self.__setup_globals ()

	def __setup_globals (self):
		"""Initializes all globals used for quick referencing.

		Will only be called by the first Actor created."""

		global classcount, dualswap
		classcount = CommonTables.Classes.GetRowCount ()
		dualswap = [0]*classcount

		for i in range(classcount):
			rowname = CommonTables.Classes.GetRowName(i)
			classid = CommonTables.Classes.GetValue (rowname, "ID")
			classnames = rowname.split("_")

			#set the MC_WAS_ID of the first class
			if len(classnames) == 2:
				dualswap[classid-1] = CommonTables.Classes.GetValue (rowname, "MC_WAS_ID")

	def Classes (self):
		"""Returns a list with all the class IDs."""

		if self.__classes == None:
			#already reversed in ClassNames
			self.__classes = [CommonTables.Classes.GetValue (name, "ID", GTV_INT) for name in self.ClassNames()]
		return self.__classes

	def ClassNames (self):
		"""Returns a list will all the class names."""

		if self.__classnames == None:
			self.__classnames = GUICommon.GetClassRowName (self.classid, "class").split("_")
			if self.IsDualSwap():
				self.__classnames.reverse()
		return self.__classnames

	def ClassTitle (self):
		"""Returns the class title as a displayable string."""

		if self.__classtitle != None:
			return self.__classtitle

		self.__classtitle = GemRB.GetPlayerStat (self.pc, IE_TITLE1)
		self.ClassNames()
		if self.__classtitle == 0:
			if self.multiclass and self.isdual == 0:
				self.__classtitle = CommonTables.Classes.GetValue ("_".join(self.__classnames), "CAP_REF", GTV_REF)
			elif self.isdual:
				# first (previous) kit or class of the dual class
				self.Classes()
				if self.KitIndex():
					self.__classtitle = CommonTables.KitList.GetValue (self.__kitindex, 2, GTV_REF)
				else:
					self.__classtitle = CommonTables.Classes.GetValue (self.__classnames[1], "CAP_REF", GTV_REF)
				self.__classtitle = self.__classtitle + " / " + \
					CommonTables.Classes.GetValue (self.__classnames[0], "CAP_REF", GTV_REF)
			else: # ordinary class or kit
				if self.KitIndex():
					self.__classtitle = CommonTables.KitList.GetValue (self.__kitindex, 2, GTV_REF)
				else:
					self.__classtitle = CommonTables.Classes.GetValue ("_".join(self.__classnames), "CAP_REF", GTV_REF)

		if self.__classtitle == "*":
			self.__classtitle = 0
		return self.__classtitle

	def IsDualSwap (self):
		"""Returns true if IE_LEVEL is opposite of expectations."""

		if self.__dualswap == None:
			self.__dualswap = (self.isdual & CommonTables.Classes.GetValue \
				(self.ClassNames()[0], "MC_WAS_ID", GTV_INT)) > 0
		return self.__dualswap

	def KitIndex (self):
		"""Returns the kit index in relation to kitlist.2da."""

		if self.__kitindex != None:
			return self.__kitindex

		Kit = GemRB.GetPlayerStat (self.pc, IE_KIT)
		self.__kitindex = 0

		if Kit & 0xc000 == 0x4000:
			self.__kitindex = Kit & 0xfff

		# carefully looking for kit by the usability flag
		# since the barbarian kit id clashes with the no-kit value
		if self.__kitindex == 0 and Kit != 0x4000:
			self.__kitindex = CommonTables.KitList.FindValue (6, Kit)
			if self.__kitindex is None:
				self.__kitindex = 0

		return self.__kitindex

	def LevelDiffs (self):
		"""Returns the differences between the current and next classes."""
		return [(next-current) for current,next in zip(self.Levels(),
			self.NextLevels())]

	def Levels (self):
		"""Returns the current level of each class."""
		if self.__levels == None:
			self.__levels = [level for slot in levelslots for level \
				in [GemRB.GetPlayerStat (self.pc, slot)] if level>0]
			if self.IsDualSwap():
				self.__levels.reverse()
		return self.__levels

	def NextLevelExp (self):
		"""Returns the experience required to level each class."""

		#filtering the old dual class out seems unnecessary
		#just be sure to use NumClasses() or isdual to check
		return [CommonTables.NextLevel.GetValue (name, str(level+1)) for name,level \
			in zip(self.ClassNames(), self.Levels())]

	def NextLevels (self):
		"""Returns the next level for each class."""

		if self.__nextlevels != None:
			return self.__nextlevels

		xp = GemRB.GetPlayerStat (self.pc, IE_XP) // self.NumClasses()

		self.__nextlevels = []
		for name, level in zip(self.ClassNames(), self.Levels() ):
			nextLevel = level

			#we only want the current level for the old part of a dual-class
			if len(self.__nextlevels) < self.__numclasses:
				for current in range(level+1, CommonTables.NextLevel.GetColumnCount () ):
					if CommonTables.NextLevel.GetValue (name, str(current)) <= xp:
						nextLevel = current
					else:
						break
			self.__nextlevels.append(nextLevel)

		return self.__nextlevels
	
	def NumClasses (self):
		"""Returns the number of *active* classes."""
		if self.__numclasses == None:
			if self.isdual:
				self.__numclasses = 1
			else:
				self.__numclasses = len(self.ClassNames() )
		return self.__numclasses


	def RaceName (self):
		"""Returns the race string."""
		pass

	def Reset (self, pc):
		"""Resets all internal variables.

		This should be called after any fundamental changes to the pc.
		This includes: dualclassing, leveling."""

		#accessible variables
		self.pc = pc
		self.classid = GemRB.GetPlayerStat (self.pc, IE_CLASS)
		self.isdual = GemRB.GetPlayerStat (self.pc, IE_MC_FLAGS) & MC_WAS_ANY_CLASS
		self.multiclass = CommonTables.Classes.GetValue (GUICommon.GetClassRowName (pc), "MULTI")

		# internal variables - these are only initialized on the first
		#call to their respective function, and stored thereafter
		self.__classes = None
		self.__classnames = None
		self.__classtitle = None
		self.__dualswap = None
		self.__kitindex = None
		self.__levels = None
		self.__nextlevels = None
		self.__numclasses = None



# GemRB - Infinity Engine Emulator
# Copyright (C) 2003 The GemRB Project
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
#
#
# common character generation display code
import GemRB
import GUICommon
from ie_stats import *
from GUIDefines import *

class CharGen:
	def __init__(self,stages,startText,importFn,resetButton=False):
		"""Sets up a character generation system.
		slot: 	index of slot for which to generate 
		stages: 	List stages
		startText:		text to set info field in stage 0
		importFn: 	function ran when import is pressed
		resetButton:	whether or not to show a reset button
		
		where each stage is defined by either an intermediate screen: 
			name: 		name
			control: 		id of the control
			text: 		id of button text or string to use
		or a request stage
			name:			name
			script | setFn:		script name to proceed or function to request and set data for this stage
			commentFn(area):	function to append information to the text area
			unsetFn:			function to remove data for this stage
			guard			return whether or not to activate this stage
		"""
		self.stages = stages
		self.imp = importFn
		self.showReset = resetButton
		self.step = 0
		self.window = None
		self.startText=startText
	
	def displayOverview(self):
		"""
		Sets up the primary character generation window.
		show all comments of previous stages		
		"""
		if(self.window): 
			CharGenWindow = self.window
		else:
			CharGenWindow = GemRB.LoadWindow (0, "GUICG")
	
		step = self.step
		
		#set portrait	
		PortraitButton = CharGenWindow.GetControl (12)
		PortraitButton.SetFlags(IE_GUI_BUTTON_PICTURE|IE_GUI_BUTTON_NO_IMAGE,OP_SET)
		PortraitName = GemRB.GetToken ("LargePortrait")
		PortraitButton.SetPicture (PortraitName, "NOPORTLG")
		PortraitButton.SetState(IE_GUI_BUTTON_LOCKED)
		
		#set stage buttons
		i = 0
		for stage in self.stages:
			if(len(stage) == 3): #short stage
				(name,control,text) = stage
				button = CharGenWindow.GetControl(control)
				button.SetText(text);
				if i == step:
					button.SetState(IE_GUI_BUTTON_ENABLED)
					button.MakeDefault()
					button.OnPress (lambda: self.next())
				else:
					button.SetState(IE_GUI_BUTTON_DISABLED)
			i = i + 1
		
		#set back button	
		BackButton = CharGenWindow.GetControl (11)
		#BackButton.SetText (15416)
		if(self.step != 0):
			BackButton.SetState (IE_GUI_BUTTON_ENABLED)
		else:
			BackButton.SetState(IE_GUI_BUTTON_DISABLED)
		BackButton.OnPress (lambda: self.back())
		BackButton.MakeEscape()

		AcceptButton = CharGenWindow.GetControl (8)
		playmode = GemRB.GetVar ("PlayMode")
		if playmode is not None:
			AcceptButton.SetText (11962)
		else:
			AcceptButton.SetText (13956)

		#set scrollbar
		ScrollBar = CharGenWindow.GetControl (10)
		CharGenWindow.SetEventProxy(ScrollBar)
		
		#set import
		ImportButton = CharGenWindow.GetControl (13)
		ImportButton.SetText (13955)
		ImportButton.SetState (IE_GUI_BUTTON_ENABLED)
		ImportButton.OnPress (lambda: self.imprt())

		#set cancel and start over
		CancelButton = CharGenWindow.GetControl (15)
		if step == 0 or not self.showReset:
			CancelButton.SetText (13727) # Cancel
		else:
			CancelButton.SetText (8159) # Start over
		CancelButton.SetState (IE_GUI_BUTTON_ENABLED)
		CancelButton.OnPress (lambda: self.cancel())

		#set and fill overview
		TextAreaControl = CharGenWindow.GetControl (9)
		if(self.step == 0):
			TextAreaControl.SetText(self.startText)
		else:
			TextAreaControl.SetText("")
		for part in range(step):
			if(len(self.stages[part]) == 5):
				(name,setFn,commentFn,unsetFn,guard) = self.stages[part]
				if(commentFn != None):
					commentFn(TextAreaControl)
		
		#show
		CharGenWindow.Focus()
		self.window = CharGenWindow
	
	def unset(self,stage):
		if(len(self.stages[stage]) == 5):
			(name,setFn,commentFn,unsetFn,guard) = self.stages[stage]
			if(unsetFn != None):
				unsetFn()
	#set next script to for step, return false if the current step should be skipped
	def setScript(self):
		if(len(self.stages[self.step]) == 5): #long record: script of function
			(name,setFn,commentFn,unsetFn,guardFn) = self.stages[self.step]
			if(guardFn and not guardFn()):
				return False
			if(hasattr(setFn, "__call__")):
				setFn()
			else:
				GemRB.SetNextScript(setFn)
		else: #short record: overview
			GemRB.SetNextScript ("CharGen")
		return True
		
	def cancel(self):
		"""Revert back to the first step; unset all actions."""

		for i in range(self.step,-1,-1):
			self.unset(i)

		# if required back to main screen, otherwise reloop
		if self.step == 0 or not self.showReset:	
			self.window.Close ()
			self.window = None
			GemRB.SetNextScript ("Start")
		else:
			GemRB.SetNextScript ("CharGen")

		self.step = 0


	def imprt(self):
		"""Opens the character import window."""
		self.imp()

	def back(self):
		"""Moves to the previous step. Unsets last"""
		self.unset(self.step)
		if len(self.stages[self.step]) == 3:
			#short, return to other short
			self.step = self.step - 1
			self.unset(self.step)
			while len(self.stages[self.step]) != 3:
				self.step = self.step - 1
				self.unset(self.step)
		else:
			self.step = self.step - 1
			self.unset(self.step)

		while(not self.setScript()):
			self.step = self.step - 1
			self.unset(self.step)

	
	def next(self):
		"""Calls the next setter."""
		self.step = self.step + 1
		while(not self.setScript()):
			self.step = self.step + 1

	def close(self):
		if(self.window):
			self.window.Close ()
	
	def jumpTo(self,to):
		if type(to) == str:
			done = False
			for i in range(len(self.stages)):
				if(to == self.stages[i][0]):
					self.step = i
					done = True
			if(not done):
				raise ValueError("stage name not found: "+str(to))
		elif type(to) == int:
			if(to<0):
				to = len(self.stages)+to
			if(to<0 or to>=len(self.stages)):
				raise ValueError("stage index not found: "+str(to))
			self.step = to
			
			return
		else:
			raise ValueError("bad arg type: "+str(type(to)) + " " + str(to))
		while(not self.setScript()):
			self.step = self.step + 1

CharGenMaster = None

def back(Window=None):
	"""Moves to the previous step."""
	global CharGenMaster
	CharGenMaster.back()

	if (Window):
		Window.Close()

def next():
	"""Moves to the next step."""
	global CharGenMaster
	CharGenMaster.next()

def close():
	"""Terminates CharGen."""
	global CharGenMaster
	CharGenMaster.close()
	CharGenMaster = None

def jumpTo(stage):
	global CharGenMaster
	CharGenMaster.jumpTo(stage)




# GemRB - Infinity Engine Emulator
# Copyright (C) 2003-2005 The GemRB Project
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
#

###################################################

import GemRB
import GameCheck
import GUICommon
import MessageWindow
from ie_restype import RES_2DA

def EnterGame():
	GemRB.GameSetPartySize (GameCheck.MAX_PARTY_SIZE)
	GemRB.GameSetProtagonistMode(1)
	GUICommon.SetSaveDir ()

	MessageWindow.OnLoad()

	# reset gold, so a message is printed
	if GemRB.GetGameTime () <= 2103:
		GemRB.SetTimedEvent (lambda: GemRB.GameSetPartyGold (GemRB.GameGetPartyGold (), 1), 1)

#upgrade savegame to next version
#this is not necessary if TotSC was already installed before starting the game
def GameExpansion():
	if GameCheck.HasTOTSC ():
		#reload world map if it doesn't have AR1000
		GemRB.UpdateWorldMap("WORLDMAP", "AR1000")



# GemRB - Infinity Engine Emulator
# Copyright (C) 2003 The GemRB Project
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
#


# GUIW.py - scripts to control some windows from the GUIWORLD winpack
# except of Actions, Portrait, Options and Dialog windows
#################################################################

import GemRB

import ActionsWindow
import CommonWindow
import Container
import GameCheck
import GUICommon
import GUICommonWindows
from GUIDefines import *

ContinueWindow = None

def OpenDialogButton(id):
	window = GemRB.LoadWindow (id, GUICommon.GetWindowPack())
	window.SetFlags (IE_GUI_VIEW_IGNORE_EVENTS, OP_OR)

	MsgWin = GemRB.GetView("MSGWIN")

	frame = MsgWin.GetFrame()
	offset = 0
	if GameCheck.IsGemRBDemo ():
		offset = window.GetFrame()['h']
	window.SetPos(frame['x'], frame['y'] + frame['h'] - offset)

	# Dialog button shouldnt be in front
	win = GemRB.GetView("OPTWIN")
	if win:
		win.Focus()
	win = GemRB.GetView("PORTWIN")
	if win:
		win.Focus()

	return window

def DialogStarted ():
	global ContinueWindow

	# try to force-close anything which is open
	GUICommonWindows.CloseTopWindow()
	Container.CloseContainerWindow()

	# opening control size to maximum, enabling dialog window
	CommonWindow.SetGameGUIHidden(False)
	GemRB.GameSetScreenFlags(GS_DIALOG, OP_OR)

	ContinueWindow = OpenDialogButton(9)

def DialogEnded ():
	global ContinueWindow

	ActionsWindow.UpdateActionsWindow ()
	if not ContinueWindow:
		return

	# reset the global hotkey, so it doesn't interfere with console use afterwards
	ContinueWindow.GetControl(0).SetHotKey (None, 0, True)
	ContinueWindow.Close ()
	ContinueWindow = None

def CloseContinueWindow ():
	# don't close the actual window now to avoid flickering: we might still want it open
	GemRB.SetVar ("DialogChoose", GemRB.GetVar ("DialogOption"))

def NextDialogState ():
	if not ContinueWindow:
		return

	ContinueWindow.GetControl(0).SetVisible(False)
	ContinueWindow.GetControl(0).SetDisabled(True)

def OpenEndMessageWindow ():
	Button = ContinueWindow.GetControl (0)
	Button.SetVisible(True)
	Button.SetDisabled(False)
	EndDLGStrref = 9371
	if GameCheck.IsGemRBDemo ():
		EndDLGStrref = 67
	Button.SetText (EndDLGStrref)
	Button.OnPress (CloseContinueWindow)
	Button.SetFlags (IE_GUI_BUTTON_NO_TOOLTIP, OP_OR)
	Button.MakeDefault(True)

def OpenContinueMessageWindow ():
	#continue
	Button = ContinueWindow.GetControl (0)
	Button.SetVisible(True)
	Button.SetDisabled(False)
	ContinueStrref = 9372
	if GameCheck.IsGemRBDemo ():
		ContinueStrref = 66
	Button.SetText (ContinueStrref)
	Button.OnPress (CloseContinueWindow)
	Button.SetFlags (IE_GUI_BUTTON_NO_TOOLTIP, OP_OR)
	Button.MakeDefault(True)

def DeathWindow ():
	# break out of any cutscenes, so we get mouse input back
	# can also happen in "cutscene lite" mode, eg. with wk spirit warrior dream #715
	GemRB.EndCutSceneMode ()
	GemRB.ExecuteString ("SetCursorState(0)")
	if GameCheck.IsIWD1():
		#no death movie, but music is changed
		GemRB.LoadMusicPL ("Theme.mus",1)
	CommonWindow.SetGameGUIHidden(True)
	GemRB.SetTimedEvent (DeathWindowEnd, 10)
	return

def DeathWindowEnd ():
	#playing death movie before continuing
	if not GameCheck.IsIWD1():
		GemRB.PlayMovie ("deathand",1)
	GemRB.GamePause (1,3)

	Window = GemRB.LoadWindow (17, GUICommon.GetWindowPack(), WINDOW_BOTTOM|WINDOW_HCENTER)

	#reason for death
	Label = Window.GetControl (0x0fffffff)
	Label.SetText (16498)

	#load
	Button = Window.GetControl (1)
	Button.SetText (15590)
	Button.OnPress (LoadPress)

	#quit
	Button = Window.GetControl (2)
	Button.SetText (15417)
	Button.OnPress (QuitPress)

	Window.ShowModal (MODAL_SHADOW_GRAY)
	return

def QuitPress():
	GemRB.QuitGame ()
	GemRB.SetNextScript ("Start")
	return

def LoadPress():
	GemRB.QuitGame ()
	GemRB.SetNextScript ("GUILOAD")
	return




# GemRB - Infinity Engine Emulator
# Copyright (C) 2003-2004 The GemRB Project
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
#


# GUIPR.py - scripts to control the priest spells windows from the GUIPR winpack

###################################################

import GemRB
import GameCheck
import GUICommon
import GUICommonWindows
import CommonTables
import Spellbook
from GUICommon import BindControlCallbackParams
from GUIDefines import *
from ie_stats import *
from ie_action import ACT_CAST

PriestSpellWindow = None
PriestSpellLevel = 0

FlashResRef = "FLASHBR" if GameCheck.IsBG2OrEE () else "FLASH"

def InitPriestWindow (Window):
	global PriestSpellWindow
	PriestSpellWindow = Window
	
	Button = Window.GetControl (1)
	Button.OnPress (PriestPrevLevelPress)

	Button = Window.GetControl (2)
	Button.OnPress (PriestNextLevelPress)

	#setup level buttons
	if GameCheck.IsBG2OrEE ():
		for i in range (7):
			Button = Window.GetControl (55 + i)
			Button.OnPress (RefreshPriestLevel)
			Button.SetFlags (IE_GUI_BUTTON_RADIOBUTTON, OP_OR)

		for i in range (7):
			Button = Window.GetControl (55 + i)
			Button.SetVarAssoc ("PriestSpellLevel", i)

	# Setup memorized spells buttons
	for i in range (12):
		Button = Window.GetControl (3 + i)
		color = {'r' : 0, 'g' : 0, 'b' :0, 'a' : 64}
		Button.SetBorder (0,color,0,1)
		Button.SetSprites ("SPELFRAM",0,0,0,0,0)
		Button.SetState (IE_GUI_BUTTON_LOCKED)
		Button.SetVarAssoc ("Memorized", i)
		Button.SetAnimation (None)
		# reposition from a 4x3 grid to the bottom
		if GameCheck.IsBG2EE ():
			row = i // 4
			col = i % 4
			width = 54 + 8 # slot + padding
			Button.SetPos (100 + row * width * 4 + col * width, 644)
			# the bam isn't centered, so let's cheat
			Button.SetFlags (IE_GUI_BUTTON_NO_IMAGE, OP_OR)

	# Setup book spells buttons
	for i in range (GUICommon.GetGUISpellButtonCount()):
		Button = Window.GetControl (27 + i)
		Button.SetFlags (IE_GUI_BUTTON_NO_IMAGE, OP_OR)
		Button.SetState (IE_GUI_BUTTON_LOCKED)

	return

def UpdatePriestWindow (Window):
	pc = GemRB.GameGetSelectedPCSingle ()
	spelltype = IE_SPELL_TYPE_PRIEST
	level = PriestSpellLevel
	max_mem_cnt = GemRB.GetMemorizableSpellsCount (pc, spelltype, level)
	
	ClassName = GUICommon.GetClassRowName (pc)
	DivineCaster = CommonTables.ClassSkills.GetValue (ClassName, "CLERICSPELL")
	if DivineCaster == "*":
		# also check the DRUIDSPELL column
		DivineCaster = CommonTables.ClassSkills.GetValue (ClassName, "DRUIDSPELL")
	CantCast = DivineCaster == "*"
	CantCast += GemRB.GetPlayerStat(pc, IE_DISABLEDBUTTON)&(1<<ACT_CAST)

	GUICommon.AdjustWindowVisibility (Window, pc, CantCast)

	Label = Window.GetControl (0x10000032)
	# bg2 uses a shorthand form
	if GameCheck.IsBG2OrEE ():
		GemRB.SetToken ("SPELLLEVEL", str(level+1))
		Label.SetText (10345)
	else:
		GemRB.SetToken ('LEVEL', str(level+1))
		Label.SetText (12137)

	Name = GemRB.GetPlayerName (pc, 0)
	Label = Window.GetControl (0x10000035)
	Label.SetText (Name)

	Sorcerer = Spellbook.HasSorcererBook (pc)
	mem_cnt = GemRB.GetMemorizedSpellsCount (pc, spelltype, level, False)
	for i in range (12):
		Button = Window.GetControl (3 + i)
		Button.SetFlags (IE_GUI_VIEW_INVISIBLE | IE_GUI_VIEW_DISABLED, OP_OR if Sorcerer else OP_NAND)
		if Sorcerer:
			continue

		if i < mem_cnt:
			ms = GemRB.GetMemorizedSpell (pc, spelltype, level, i)
			Button.SetSpellIcon (ms['SpellResRef'], 0)
			if not GameCheck.IsBG2EE ():
				Button.SetFlags (IE_GUI_BUTTON_PICTURE, OP_SET)
			if ms['Flags']:
				Button.OnPress (OpenPriestSpellUnmemorizeWindow)
			else:
				Button.OnPress (OnPriestUnmemorizeSpell)
			spell = GemRB.GetSpell (ms['SpellResRef'])
			Button.OnRightPress (BindControlCallbackParams(OpenPriestSpellInfoWindow, spell))
			Button.SetTooltip (spell['SpellName'])
			Button.EnableBorder (0, ms['Flags'] == 0)
		else:
			if i < max_mem_cnt:
				Button.SetFlags (IE_GUI_BUTTON_NORMAL, OP_SET)
			else:
				Button.SetFlags (IE_GUI_BUTTON_NO_IMAGE, OP_SET)
			Button.OnPress (None)
			Button.OnRightPress (None)
			Button.SetTooltip ('')
			Button.EnableBorder (0, 0)

	known_cnt = GemRB.GetKnownSpellsCount (pc, spelltype, level)
	btncount = GUICommon.GetGUISpellButtonCount()
	i = 0
	for i in range (known_cnt):
		Button = Window.GetControl (27 + i)
		ks = GemRB.GetKnownSpell (pc, spelltype, level, i)
		Button.SetSpellIcon (ks['SpellResRef'], 0)
		if not GameCheck.IsBG2EE ():
			Button.SetFlags (IE_GUI_BUTTON_NO_IMAGE, OP_NAND)
		Button.OnPress (OnPriestMemorizeSpell)
		spell = GemRB.GetSpell (ks['SpellResRef'])
		Button.OnRightPress (BindControlCallbackParams(OpenPriestSpellInfoWindow, spell))
		Button.SetTooltip (spell['SpellName'])
		Button.SetVarAssoc ("Memorized", i)

	if known_cnt == 0: i = -1
	for i in range (i + 1, btncount):
		Button = Window.GetControl (27 + i)
		
		Button.SetFlags (IE_GUI_BUTTON_NO_IMAGE, OP_OR)
		Button.SetFlags (IE_GUI_BUTTON_PICTURE, OP_NAND)
		Button.OnPress (None)
		Button.OnRightPress (None)
		Button.SetTooltip ('')
		Button.EnableBorder (0, 0)

	Window.Focus()
	return

TogglePriestWindow = GUICommonWindows.CreateTopWinLoader(2, "GUIPR", GUICommonWindows.ToggleWindow, InitPriestWindow, UpdatePriestWindow, True)
OpenPriestWindow = GUICommonWindows.CreateTopWinLoader(2, "GUIPR", GUICommonWindows.OpenWindowOnce, InitPriestWindow, UpdatePriestWindow, True)

def PriestPrevLevelPress ():
	global PriestSpellLevel

	if PriestSpellLevel > 0:
		PriestSpellLevel = PriestSpellLevel - 1
		UpdatePriestWindow (PriestSpellWindow)
	return

def PriestNextLevelPress ():
	global PriestSpellLevel

	if PriestSpellLevel < 6:
		PriestSpellLevel = PriestSpellLevel + 1
		UpdatePriestWindow (PriestSpellWindow)
	return

def RefreshPriestLevel ():
	global PriestSpellLevel

	PriestSpellLevel = GemRB.GetVar ("PriestSpellLevel")
	UpdatePriestWindow (PriestSpellWindow)
	return

def OpenPriestSpellInfoWindow (spell):
	Window = GemRB.LoadWindow (3, "GUIPR")

	#back
	Button = Window.GetControl (5)
	Button.SetText (15416)
	Button.OnPress (Window.Close)

	if GameCheck.IsBG2OrEE ():
		Label = Window.GetControl (0x0fffffff)
	else:
		Label = Window.GetControl (0x10000000)
	Label.SetText (spell['SpellName'])

	Button = Window.GetControl (2)
	Button.SetSpellIcon (spell['SpellResRef'], 1)

	Text = Window.GetControl (3)
	Text.SetText (spell['SpellDesc'])

	Window.ShowModal (MODAL_SHADOW_GRAY)
	return

def OnPriestMemorizeSpell (btn):
	pc = GemRB.GameGetSelectedPCSingle ()
	level = PriestSpellLevel
	spelltype = IE_SPELL_TYPE_PRIEST
	Window = btn.Window

	def Complete():
		mem_cnt = GemRB.GetMemorizedSpellsCount (pc, spelltype, level, False)
		AnimBtn = Window.GetControl(mem_cnt + 2)
		AnimBtn.SetAnimation(FlashResRef, 0, A_ANI_PLAYONCE | A_ANI_BLEND)
		AnimBtn.OnAnimEnd(lambda: UpdatePriestWindow(Window))
		UpdatePriestWindow(Window)

	index = btn.Value
	if GemRB.MemorizeSpell (pc, spelltype, level, index):
		GemRB.PlaySound ("GAM_24")
		Button = PriestSpellWindow.GetControl(index + 27)
		Button.SetAnimation (FlashResRef, 0, A_ANI_PLAYONCE | A_ANI_BLEND)
		Button.OnAnimEnd(Complete)

	return

def OpenPriestSpellRemoveWindow ():
	Window = GemRB.LoadWindow (5, "GUIPR")

	# "Are you sure you want to ....?"
	TextArea = Window.GetControl (3)
	TextArea.SetText (11824)

	# Remove
	Button = Window.GetControl (0)
	Button.SetText (17507)
	def RemoveSpell (btn):
		OnPriestRemoveSpell(btn)
		Window.Close()

	Button.OnPress (RemoveSpell)
	Button.MakeDefault()

	# Cancel
	Button = Window.GetControl (1)
	Button.SetText (13727)
	Button.OnPress (Window.Close)
	Button.MakeEscape()

	Window.ShowModal (MODAL_SHADOW_GRAY)
	return

def OpenPriestSpellUnmemorizeWindow (btn):
	Window = GemRB.LoadWindow (5, "GUIPR")

	# "Are you sure you want to ....?"
	TextArea = Window.GetControl (3)
	TextArea.SetText (11824)

	# Remove
	Button = Window.GetControl (0)
	Button.SetText (17507)
	def Unmemorize (btn):
		OnPriestUnmemorizeSpell (btn)
		Window.Close()
	
	Button.SetValue (btn.Value)
	Button.OnPress (Unmemorize)
	Button.MakeDefault()

	# Cancel
	Button = Window.GetControl (1)
	Button.SetText (13727)
	Button.OnPress (Window.Close)
	Button.MakeEscape()

	Window.ShowModal (MODAL_SHADOW_GRAY)
	return

def OnPriestUnmemorizeSpell (btn):
	pc = GemRB.GameGetSelectedPCSingle ()
	level = PriestSpellLevel
	spelltype = IE_SPELL_TYPE_PRIEST
	index = btn.Value

	if GemRB.UnmemorizeSpell (pc, spelltype, level, index):
		GemRB.PlaySound ("GAM_44")
		Button = PriestSpellWindow.GetControl(index + 3)
		Button.SetAnimation (FlashResRef, 0, A_ANI_PLAYONCE | A_ANI_BLEND)
		Button.OnAnimEnd(lambda: UpdatePriestWindow (PriestSpellWindow))
	return

def OnPriestRemoveSpell (btn):
	pc = GemRB.GameGetSelectedPCSingle ()
	level = PriestSpellLevel
	spelltype = IE_SPELL_TYPE_PRIEST

	index = btn.Value

	#remove spell from memory
	GemRB.RemoveSpell (pc, spelltype, level, index)
	UpdatePriestWindow(btn)
	return

###################################################
# End of file GUIPR.py
